package brain;

import java.util.Random;

/**
 * This module is creating different kinds of noise: white, pink, brown,
 * gaussian. It is to be used as an afferent input module by other modules.
 * 
 * @author Mathias Linkerhand
 * @version 2011-08-25
 */
public class NoiseModule extends Module {
  private static final long serialVersionUID = 1L;

  private double            changeInterval;
  private double            lastChange       = 0;
  private Random            random;
  private int               noise;

  /**
   * White noise is a signal that contains all frequencies in equal amounts.
   */
  public static final int   WHITE_NOISE      = 0;
  public static final int   PINK_NOISE       = 1;
  public static final int   BROWN_NOISE      = 2;
  public static final int   GAUSSIAN_NOISE   = 3;

  public NoiseModule() {
    this(WHITE_NOISE, 1.0, new Random(0));
  }

  public NoiseModule(final int noise, final double changeInterval, final Random random) {
    this.noise = noise;
    this.setChangeInterval(changeInterval);
    this.setRandom(random);

    this.setSize(1);
  }

  @Override
  protected void calculateOneTimeStep(double timeDelta) {
    if (this.elapsedTime > this.lastChange + this.getChangeInterval()) {
      for (int neuron = 0; neuron < this.getSize(); neuron++) {
        switch (this.noise) {
          case WHITE_NOISE:
            this.setActivity(neuron, this.random.nextDouble() + this.getBias(neuron));
            break;
          case PINK_NOISE:
            // TODO: Implement!
            throw new UnsupportedOperationException();
            //break;
          case BROWN_NOISE:
            // TODO: Implement!
            throw new UnsupportedOperationException();
            //break;
          case GAUSSIAN_NOISE:
            this.setActivity(neuron, this.random.nextGaussian() + this.getBias(neuron));
            break;
          default:
            throw new IllegalStateException("wrong noise specification");
        }
      }
      this.lastChange = this.elapsedTime;
    }
  }

  public double getChangeInterval() {
    return this.changeInterval;
  }

  public void setChangeInterval(double changeInterval) {
    this.changeInterval = changeInterval;
  }

  public Random getRandom() {
    return this.random;
  }

  public void setRandom(final Random random) {
    this.random = random;
  }
}