package brain.cliqueCalculation;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeSet;

public class CliqueDistribution implements Comparable<CliqueDistribution> {
  private int                         cliqueCount  = 0;
  protected HashMap<Integer, Integer> distribution = null;

  public CliqueDistribution() {
    super();
    distribution = new HashMap<Integer, Integer>();
  }

  public HashMap<Integer, Integer> getDistibution() {
    return distribution;
  }

  //	public void setDistibution(HashMap<Integer, Integer> distibution) {
  //		this.distribution = distibution;
  //	}

  public void add(Integer object) {
    cliqueCount++;
    if (distribution.get(object) == null) {
      distribution.put(object, Integer.valueOf(1));
    } else {
      distribution.put(object, (distribution.get(object) + 1));
    }
  }

  public void printCliqueDistribution() {
    System.out.println("Clique Distribution");
    Iterator<Integer> DistributionIterator = new TreeSet<Integer>(distribution.keySet()).iterator();

    Integer lauf = null;
    while (DistributionIterator.hasNext()) {
      lauf = DistributionIterator.next();
      System.out.println(lauf + ", " + distribution.get(lauf));
    }
  }

  /**
   * Compares this object with the specified object for order. Returns a
   * negative integer or zero, as this object is less than or equal to, than
   * the specified object.
   * 
   * 
   * @param o
   * object to be compared @
   * @return negative integer, or zero, as this object is less than or equal
   *         to, than the specified object.
   */

  @Override
  public int compareTo(CliqueDistribution o) {
    if (distribution.size() != o.getDistibution().size()) {
      return -1;
    }

    Iterator<Integer> oIterator = new TreeSet<Integer>(o.getDistibution().keySet()).iterator();
    Iterator<Integer> DistributionIterator = new TreeSet<Integer>(distribution.keySet()).iterator();

    Integer lauf1 = null;
    Integer lauf2 = null;
    while (DistributionIterator.hasNext()) {
      lauf1 = DistributionIterator.next();
      lauf2 = oIterator.next();
      if (lauf2 != lauf1) {
        return -1;
      }
      // System.out.println(lauf + ", " + distribution.get(lauf));
      else if (Integer.parseInt(o.getDistibution().get(lauf2).toString()) != Integer.parseInt(distribution.get(lauf1).toString())) {
        return -1;
      }
    }
    return 0;
  }

  public void writeAbsoluteCliqueDistributionToGnuPlot(String path, String tmp) {
    File f = new File(path);
    FileWriter fw;
    BufferedWriter bfw;
    try {
      fw = new FileWriter(f);
      bfw = new BufferedWriter(fw);
      TreeSet<Integer> sortedKeySet = new TreeSet<Integer>(distribution.keySet());
      Iterator<Integer> iteratorSortedKeySet = sortedKeySet.iterator();
      Integer key, value;
      bfw.write(tmp);
      while (iteratorSortedKeySet.hasNext()) {
        key = iteratorSortedKeySet.next();
        value = distribution.get(key);
        bfw.write(key + " " + value + "\n");
      }
      bfw.close();
    } catch (IOException e) {
      e.printStackTrace();
    }
    //		System.out.println("Clique Distribution");
    //		Iterator DistributionIterator = new TreeSet<Integer>(distribution
    //				.keySet()).iterator();
    //
    //		Integer lauf = null;
    //		while (DistributionIterator.hasNext()) {
    //			lauf = (Integer) DistributionIterator.next();
    //			System.out.println(lauf + ", " + distribution.get(lauf));
    //		}	
  }

  public void writeRelativeCliqueDistributionToGnuPlot(String path, String tmp) {
    File f = new File(path);
    FileWriter fw;
    BufferedWriter bfw;
    int sum = 0;
    try {
      fw = new FileWriter(f);
      bfw = new BufferedWriter(fw);
      TreeSet<Integer> sortedKeySet = new TreeSet<Integer>(distribution.keySet());
      Iterator<Integer> iteratorSortedKeySet = sortedKeySet.iterator();
      Integer key, value;
      bfw.write(tmp);
      while (iteratorSortedKeySet.hasNext()) {
        key = iteratorSortedKeySet.next();
        value = distribution.get(key);
        sum += value;
      }
      bfw.close();
    } catch (IOException e) {
      e.printStackTrace();
    }
    try {
      fw = new FileWriter(f);
      bfw = new BufferedWriter(fw);
      TreeSet<Integer> sortedKeySet = new TreeSet<Integer>(distribution.keySet());
      Iterator<Integer> iteratorSortedKeySet = sortedKeySet.iterator();
      Integer key, value;
      bfw.write(tmp);
      while (iteratorSortedKeySet.hasNext()) {
        key = iteratorSortedKeySet.next();
        value = distribution.get(key);
        bfw.write(key + " " + ((double) value / sum) + "\n");
      }
      bfw.close();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  public int getCliqueCount() {
    return cliqueCount;
  }

  public void setCliqueCount(int cliqueCount) {
    this.cliqueCount = cliqueCount;
  }
}
