package brain.cliqueCalculation;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeSet;

public class DegreeDistribution {

  private HashMap<Integer, Integer> distribution = new HashMap<Integer, Integer>();

  private int                       verticesCount;

  private Integer                   allnodes     = 0;

  // private int verticesCount = 0;

  public DegreeDistribution() {
    super();
  }

  public void add(Integer neighbourCount) {
    if (distribution.get(neighbourCount) == null) {
      distribution.put(neighbourCount, new Integer(1));
    } else {
      distribution.put(neighbourCount, distribution.get(neighbourCount) + 1);
    }
  }

  /**
   * Adds a distribution to another distribution. Returns a new distribution
   * which is a sum of both
   * 
   * @return a distribution as sum of this + dd
   */
  public DegreeDistribution addDistribution(DegreeDistribution dd) {
    TreeSet<Integer> t = new TreeSet<Integer>();
    t.addAll(dd.getDistribution().keySet());
    if (distribution != null && distribution.keySet() != null) {
      t.addAll(distribution.keySet());
    }
    Integer keys[] = new Integer[t.size()];
    t.toArray(keys);

    HashMap<Integer, Integer> result = new HashMap<Integer, Integer>();

    for (int i = 0; i < keys.length; i++) {
      Integer value1 = 0;
      Integer value2 = 0;
      if (distribution.get(keys[i]) != null) {
        value1 = distribution.get(keys[i]);
      }
      if (dd.getDistribution().get(keys[i]) != null) {
        value2 = dd.getDistribution().get(keys[i]);
      }
      if (result.get(keys[i]) == null) {
        result.put(keys[i], value1 + value2);
      } else {
        result.put(keys[i], result.get(keys[i]) + value1 + value2);
      }

    }
    DegreeDistribution resultObject = new DegreeDistribution();
    resultObject.setDistribution(result);
    return resultObject;
  }

  /**
   * Writes the absolute degree distribution to a file given by the parameter
   * path. One can give additional informations to be saved in the file in
   * parameter tmp. Every line of additional information should beginn with #
   * 
   * @param path -
   *            path of the file the distribution will be written to
   * @param tmp -
   *            additional information to be written to the file
   */
  public void writeAbsoluteDegreeDistributionToGnuPlot(String path, String tmp) {
    File f = new File(path);
    FileWriter fw;
    BufferedWriter bfw;
    try {
      fw = new FileWriter(f);
      bfw = new BufferedWriter(fw);
      TreeSet<Integer> sortedKeySet = new TreeSet<Integer>(distribution.keySet());
      Iterator<Integer> iteratorSortedKeySet = sortedKeySet.iterator();
      Integer key, value;
      bfw.write(tmp + "\n");
      bfw.write("#Vertices Count: " + verticesCount + "\n");

      while (iteratorSortedKeySet.hasNext()) {
        key = iteratorSortedKeySet.next();
        value = distribution.get(key);
        bfw.write(key + " " + value + "\n");
      }
      bfw.close();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Writes the relative degree distribution to a file given by the parameter
   * path. One can give additional informations to be saved in the file in
   * parameter tmp. Every line of additional information should beginn with #
   * 
   * @param path -
   *            path of the file the distribution will be written to
   * @param tmp -
   *            additional information to be written to the file
   */
  public void writeRelativeDegreeDistributionToGnuPlot(String path, String tmp) {
    // Sum all vertices first
    Iterator<Integer> iter = distribution.keySet().iterator();
    Integer verticesCount2 = 0;
    while (iter.hasNext()) {
      verticesCount2 += distribution.get(iter.next());
    }
    // verticesCount2=263;
    System.out.println("Vertices:" + verticesCount2);

    // assert(verticesCount2==verticesCount):"verticesCount
    // ungleich:"+verticesCount2+","+verticesCount+" in ";
    File f = new File(path);
    FileWriter fw;
    BufferedWriter bfw;
    try {
      fw = new FileWriter(f);
      bfw = new BufferedWriter(fw);
      TreeSet<Integer> sortedKeySet = new TreeSet<Integer>(distribution.keySet());
      Iterator<Integer> iteratorSortedKeySet = sortedKeySet.iterator();
      Integer key, value;
      bfw.write(tmp + "\n");
      // bfw.write("#Vertices Count: " + verticesCount2 + "\n");

      while (iteratorSortedKeySet.hasNext()) {
        key = iteratorSortedKeySet.next();
        value = distribution.get(key);
        bfw.write(key + " " + ((double) value / verticesCount2) + "\n");
      }
      bfw.close();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  public double getSlope(int from, int to) {
    // slope = (Math.log(average[this.slopeFromValue]) - Math
    // .log(average[this.slopeToValue]))
    // / (Math.log(this.slopeFromValue) - Math.log(this.slopeToValue));
    // System.out.println("slope:" + slope + "\tfrom: " +
    // this.slopeFromValue
    // + "\t to: " + this.slopeToValue);
    if (distribution.get(from) == null || distribution.get(to) == null) {
      return -1;
    }
    double fromValue = distribution.get(from) / 2601092.0;
    double toValue = distribution.get(to) / 2601092.0;

    double result = (Math.log(fromValue) - Math.log(toValue)) / (Math.log(from) - Math.log(to));
    return result;
  }

  public int getSumDegrees() {
    int sum = 0;
    Iterator<Integer> distributionSumIterator = distribution.keySet().iterator();
    while (distributionSumIterator.hasNext()) {
      sum += distribution.get(distributionSumIterator.next());

    }
    return sum;
  }

  public void printDegreeDistribution() {
    System.out.println("Degree Distribution");
    Iterator<Integer> distributionIterator = new TreeSet<Integer>(distribution.keySet()).iterator();

    Integer lauf = null;
    while (distributionIterator.hasNext()) {
      lauf = (Integer) distributionIterator.next();
      System.out.println(lauf + ", " + distribution.get(lauf));
    }
  }

  public HashMap<Integer, Integer> getDistribution() {
    return distribution;
  }

  public void setDistribution(HashMap<Integer, Integer> distribution) {
    this.distribution = distribution;
  }

  /**
   * Reads a distribution file as a flat file. format: Integer white space(s)
   * Integer. Lines beginning with # are considered a comment line and will be
   * ignored.
   * 
   * @param file
   * @return a just read degree distribution
   */
  public DegreeDistribution readFlatFile(File file) {
    FileReader fr = null;
    try {
      fr = new FileReader(file);
    } catch (FileNotFoundException e) {
      // TODO Auto-generated catch block
      // e.printStackTrace();
      System.err.println("File " + file.getAbsolutePath() + " does not exist");
    }
    BufferedReader br = new BufferedReader(fr);
    HashMap<Integer, Integer> result = new HashMap<Integer, Integer>();
    String line = "";
    int linecounter = 0;
    try {
      while ((line = br.readLine()) != null) {
        linecounter++;
        line = line.trim();
        if (line.startsWith("#") || line.indexOf(" ") == -1) {
          continue;
        }
        String firstpart = line.substring(0, line.indexOf(" ")).trim();
        String secondpart = line.substring(line.lastIndexOf(" "), line.length()).trim();
        Integer key = Integer.parseInt(firstpart);
        Integer value = Integer.parseInt(secondpart);
        allnodes = allnodes + value;
        // System.out.println(key + ", " + value);
        result.put(key, value);
      }
      System.out.println("allnodes:" + allnodes);
      allnodes = 0;
    } catch (IOException e) {
      // TODO Auto-generated catch block
      System.err.println("Error while reading " + file.getAbsolutePath());
      e.printStackTrace();
    } catch (NumberFormatException e2) {
      System.err.println("Error while parsing integer values in file " + file.getAbsolutePath() + ", line " + linecounter);
    }
    DegreeDistribution dd = new DegreeDistribution();
    dd.setDistribution(result);

    return dd;
  }

  public void setVerticesCount(int _verticesCount) {
    verticesCount = _verticesCount;
  }
}
