package brain.cliqueCalculation;

import java.math.BigDecimal;
import java.math.MathContext;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.Vector;

/**
 * @author Mathias Linkerhand
 * @version 2009-10-27
 */
public class GraphStatistics {

  private BidirectionalGraph                 graph                 = null;

  private double                             clusteringCoefficient = -1;

  private double                             averagePathLength     = -1;

  private double                             averageDegree         = -1;

  private int                                diameter              = -1;

  private HashMap<Integer, Integer>          networkDistances;

  private HashMap<Integer, Boolean>          visited;

  private boolean                            acceptIndependentSet  = true;

  private int                                cliqueCount;

  private double                             averageDegreeQuadrat  = 0;

  private Vector<BidirectionalGraph>         independentSets       = new Vector<BidirectionalGraph>();

  private HashMap<Integer, TreeSet<Integer>> visitedHashMap;

  public GraphStatistics() {
    super();

  }

  public GraphStatistics(BidirectionalGraph g) {
    super();
    this.graph = g;

  }

  /**
   * Walkes the complete graph and computes cliques
   */
  public CliqueDistribution getCliqueDistribution() {
    // Hole zweier cliquen
    HashMap<TreeSet<Integer>, TreeSet<Integer>> candidates = new HashMap<TreeSet<Integer>, TreeSet<Integer>>();

    Iterator<Integer> keySetIterator = graph.getSetOfNodes().iterator();
    Integer key = null;
    TreeSet<Integer> clique = null;
    int candidatescounter = 0;
    int tmpcounter = 0;
    int totallines = graph.getSetOfNodes().size();
    int counter = 0;
    //System.out.print("Preparing clique calculation:\t");
    while (keySetIterator.hasNext()) {
      key = keySetIterator.next();
      Iterator<Integer> nodeIterator = graph.getLinksOfNode(key).iterator();
      Integer node = null;
      while (nodeIterator.hasNext()) {
        node = (Integer) nodeIterator.next();
        clique = new TreeSet<Integer>();
        clique.add(key);
        clique.add(node);
        // if (!candidates.contains(clique)) {
        // candidates.add(clique);
        // }
        if (!candidates.containsKey(clique)) {
          candidates.put(clique, clique);
        }
      }
      if (counter == tmpcounter && totallines > 100) {
        //System.out.print(counter * 100/ (totallines - (totallines % 10)) + "% ");
        tmpcounter += (totallines - (totallines % 10)) / 10;
      }
      counter++;
    }
    //System.out.println();
    //System.out.print("calculating cliques: \t\t");
    Iterator<TreeSet<Integer>> candirator = candidates.keySet().iterator();
    candidatescounter = 0;
    tmpcounter = 0;
    while (candirator.hasNext()) {
      TreeSet<Integer> ts = new TreeSet<Integer>();
      ts = candirator.next();

      if (candidatescounter == tmpcounter && candidates.size() > 100) {
        //System.out.print(candidatescounter* 100								/ (candidates.size() - (candidates.size() % 10))								+ "% ");
        tmpcounter += (candidates.size() - (candidates.size() % 10)) / 10;
      }
      candidatescounter++;

      Iterator<Integer> ter2 = ts.iterator();
      TreeSet<Integer> allNeighboursOfCandidates = new TreeSet<Integer>();
      while (ter2.hasNext()) {
        Integer node = ter2.next();
        allNeighboursOfCandidates.addAll(graph.getLinksOfNode(node));
      }
      Iterator<Integer> ter3 = allNeighboursOfCandidates.iterator();
      boolean result = false;
      while (ter3.hasNext()) {
        Integer node = ter3.next();
        result = graph.isConnectedTo(ts, node);

        if (result)
          ts.add(node);
      }
    }
    //System.out.println();

    candirator = candidates.keySet().iterator();
    // Remove doublettes
    TreeSet<String> tst = new TreeSet<String>();
    TreeSet<Integer> tmp = new TreeSet<Integer>();

    CliqueDistribution cd = new CliqueDistribution();
    while (candirator.hasNext()) {
      tmp = candirator.next();
      String ts = tmp.toString();
      if (tst.add(ts)) {
        cd.add(tmp.size());
      }
    }
    cliqueCount = tst.size();
    return cd;
  }

  /**
   * This method calculates cliques in a graph. The idea behind that algorithm
   * is to construct cliques of two and incrementally check whether they are
   * a part of a bigger clique. First candidates are all cliques of two. To do
   * so, go through all nodes and mark all other nodes connected to them as
   * cliques of two. Check, if those cliques are part of bigger cliques, viz.
   * do the nodes of the candidate have a an extern common neighbour.
   * 
   * If they have then the candidate clique is part of a bigger clique
   * 
   * If they do not have, then the candidate clique is not a part of any
   * bigger clique, viz. is a real full interconnected subgraph - a clique
   */
  public Vector<TreeSet<Integer>> calculateCliques() {
    HashMap<TreeSet<Integer>, TreeSet<Integer>> candidates = new HashMap<TreeSet<Integer>, TreeSet<Integer>>();

    Iterator<Integer> nodeA_Iterator = graph.getSetOfNodes().iterator();
    Integer nodeA = null;
    TreeSet<Integer> clique = null;
    int candidatescounter = 0;
    int tmpcounter = 0;
    int totallines = graph.getSetOfNodes().size();
    int counter = 0;
    //System.out.print("Preparing clique calculation:\t");
    /***********************************************************************
     * Construct cliques of two
     **********************************************************************/
    while (nodeA_Iterator.hasNext()) {
      nodeA = nodeA_Iterator.next();
      Iterator<Integer> nodeB_Iterator = graph.getLinksOfNode(nodeA).iterator();
      Integer nodeB = null;
      while (nodeB_Iterator.hasNext()) {
        nodeB = nodeB_Iterator.next();
        clique = new TreeSet<Integer>();
        clique.add(nodeA);
        clique.add(nodeB);

        if (!candidates.containsKey(clique)) {
          candidates.put(clique, clique);
        }
      }
      if (counter == tmpcounter && totallines > 100) {
        //System.out.print(counter * 100/ (totallines - (totallines % 10)) + "% ");
        tmpcounter += (totallines - (totallines % 10)) / 10;
      }
      counter++;
    }
    //System.out.println();
    //System.out.print("calculating cliques: \t\t");
    Iterator<TreeSet<Integer>> allCandidateCliques = candidates.keySet().iterator();
    candidatescounter = 0;
    tmpcounter = 0;
    /**
     * For every clique check whether they are part of a bigger clique.
     * Iterate through all candidate cliques.
     */
    while (allCandidateCliques.hasNext()) {

      TreeSet<Integer> candidateClique = allCandidateCliques.next();

      if (candidatescounter == tmpcounter && candidates.size() > 100) {
        //System.out.print(candidatescounter* 100/ (candidates.size() - (candidates.size() % 10))+ "% ");
        tmpcounter += (candidates.size() - (candidates.size() % 10)) / 10;
      }
      candidatescounter++;

      Iterator<Integer> candidateCliqueNodeIterator = candidateClique.iterator();

      TreeSet<Integer> allNeighboursOfCandidates = new TreeSet<Integer>();
      /**
       * Collect all neighbours of nodes of the candidates clique 
       */
      while (candidateCliqueNodeIterator.hasNext()) {
        Integer node = candidateCliqueNodeIterator.next();
        allNeighboursOfCandidates.addAll(graph.getLinksOfNode(node));
      }

      Iterator<Integer> allNeighboursOfCandidatesiteraor = allNeighboursOfCandidates.iterator();
      /**
       * Check whether 
       */
      while (allNeighboursOfCandidatesiteraor.hasNext()) {
        Integer node = allNeighboursOfCandidatesiteraor.next();
        if (graph.isConnectedTo(candidateClique, node))
          candidateClique.add(node);
      }
    }

    //System.out.println();

    allCandidateCliques = candidates.keySet().iterator();
    /***********************************************************************
     * Remove doublettes
     **********************************************************************/
    TreeSet<String> tst = new TreeSet<String>();
    TreeSet<Integer> tmp = null;
    Vector<TreeSet<Integer>> allCliques = new Vector<TreeSet<Integer>>();

    CliqueDistribution cd = new CliqueDistribution();
    while (allCandidateCliques.hasNext()) {
      tmp = allCandidateCliques.next();
      String ts = tmp.toString();
      if (tst.add(ts)) {
        cd.add(tmp.size());
        allCliques.add(tmp);
      }
    }
    cliqueCount = tst.size();
    return allCliques;
  }

  /**
   * Computes the degree distribution
   * 
   * @return clustering coefficient
   */
  public DegreeDistribution calculateDegreeDistribution() {
    TreeSet<Integer> sortedKeySet = new TreeSet<Integer>(graph.getSetOfNodes());

    DegreeDistribution dd = new DegreeDistribution();

    Iterator<Integer> iterSortedKeySet = sortedKeySet.iterator();
    Integer tmp;
    Integer neighbourCount;
    // Integer edgesCount = 0;
    while (iterSortedKeySet.hasNext()) {
      tmp = iterSortedKeySet.next();
      neighbourCount = graph.getLinksOfNode(tmp).size();
      // edgesCount = neighbourCount;
      dd.add(neighbourCount);
    }
    return dd;
  }

  /**
   * Computes the clustering coefficient of the whole graph
   * 
   * @return clustering coefficient of the whole graph
   */

  public double clusteringCoefficient() {
    Iterator<Integer> iter = graph.getSetOfNodes().iterator();
    Double _clusteringCoefficient = 0d;
    int counter = 0;
    int tmpcounter = 0;
    int totallines = graph.getSetOfNodes().size();
    //System.out.print("Clustering Coefficient: \t");
    while (iter.hasNext()) {
      _clusteringCoefficient += clusteringCoefficient(iter.next());
      if (counter == tmpcounter && totallines > 100) {
        //System.out.print(counter * 100/ (totallines - (totallines % 10)) + "% ");
        tmpcounter += (totallines - (totallines % 10)) / 10;
      }
      counter++;
    }
    //System.out.println();
    return _clusteringCoefficient / graph.getSetOfNodes().size();
  }

  /**
   * Computes the clustering coefficient of a vertice
   * 
   * @return clustering coefficient
   */
  public double clusteringCoefficient(Integer vertice) {

    if (graph.getLinksOfNode(vertice).size() < 2)
      return 0;
    Integer neighbourArray[] = new Integer[graph.getLinksOfNode(vertice).size()];
    graph.getLinksOfNode(vertice).toArray(neighbourArray);
    int countEdges = 0;
    for (int i = 0; i < neighbourArray.length; i++) {
      for (int j = i + 1; j < neighbourArray.length; j++) {
        countEdges += graph.areNeighbours(neighbourArray[i], neighbourArray[j]) ? 1 : 0;
      }
    }

    return (double) 2 * countEdges / (neighbourArray.length * (neighbourArray.length - 1));
  }

  /**
   * Computes the average path length
   * 
   * @return average path length
   */
  public double averagePathLength() {
    double averagePathLength = 0;
    diameter = -1;

    if (graph.getNetworkTreeSet().size() == 0) {
      diameter = 0;
      return 0;

    }
    Iterator<Integer> vertices = graph.getNetworkTreeSet().keySet().iterator();
    Integer vertice = null;
    int counter = 0;
    int tmpcounter = 0;
    int totallines = graph.getNetworkTreeSet().size();
    //System.out.print("Average Pathlength:\t\t");
    visitedHashMap = new HashMap<Integer, TreeSet<Integer>>();
    while (vertices.hasNext()) {
      vertice = vertices.next();
      averagePathLength += computeDistanceFromNodeAndDiameter(vertice);
      if (counter == tmpcounter && totallines > 100) {
        //System.out.print(counter * 100/ (totallines - (totallines % 10)) + "% ");
        tmpcounter += (totallines - (totallines % 10)) / 10;
      }
      counter++;
    }
    //System.out.println();
    return averagePathLength / networkDistances.keySet().size();
  }

  /**
   * Computes the average degree
   * 
   * @return average path length
   */
  public double averageDegree() {
    Iterator<Integer> vertices = graph.getNetworkTreeSet().keySet().iterator();
    Integer vertice = null;
    int degreesum = 0;
    int counter = 0;
    int tmpcounter = 0;
    int totallines = graph.getNetworkTreeSet().size();
    double degreesumquadrat = 0;
    //System.out.print("Average Degree:\t\t\t");
    while (vertices.hasNext()) {
      vertice = vertices.next();
      degreesum += graph.getNetworkTreeSet().get(vertice).size();
      degreesumquadrat += Math.pow(graph.getNetworkTreeSet().get(vertice).size(), 2);
      if (counter == tmpcounter && totallines > 100) {
        //System.out.print(counter * 100/ (totallines - (totallines % 10)) + "% ");
        tmpcounter += (totallines - (totallines % 10)) / 10;
      }
      counter++;
    }
    //System.out.println();
    setAverageDegreeQuadrat(degreesumquadrat / graph.getNodesCount());
    return degreesum / (double) graph.getNodesCount();
  }

  /**
   * Computes the distances from a given node and returns the average distance
   * to the rest of the nodes in the graph. It also computes the diameter of
   * the graph. Putting this two calculations in one method saves time.
   * 
   * @param startVertice
   * @return average distance to the rest of the nodes in the graph
   * @see diameter
   */
  private double computeDistanceFromNodeAndDiameter(Integer startVertice) {
    /***********************************************************************
     * If there are subgraphs which are disconnected to the rest of nodes,
     * then we have a disconnected set. In that case diameter is infinite.
     * Therefore a independentSet flag will be set and the diameter will be
     * calculated anyway omitting nodes with degree of 0
     **********************************************************************/
    if (graph.getCoordinationNumber(startVertice) == 0 && acceptIndependentSet) {
      /*******************************************************************
       * Only independent sets with one vertice are detected
       ******************************************************************/
      return 0;
    }
    networkDistances = new HashMap<Integer, Integer>();
    visited = new HashMap<Integer, Boolean>();
    // visitedHashMap = new HashMap<Integer, TreeSet<Integer>>();
    //Vector<BidirectionalGraph> v = iterativeDS(startVertice);
    if (!isVisitedHashMap(startVertice)) {
      visitedHashMap.put(startVertice, new TreeSet<Integer>(visited.keySet()));
      // //System.out.println("IDS:"+visited.keySet().size());
    }
    // if (v != null) {
    // independentSets.addAll(v);
    // }
    Iterator<Integer> iter = networkDistances.keySet().iterator();
    double node_averagePathLength = 0;
    int distance = 0;
    while (iter.hasNext()) {
      distance = networkDistances.get(iter.next());
      node_averagePathLength += distance;
      diameter = distance > diameter ? distance : diameter;
    }
    return (node_averagePathLength) / (networkDistances.keySet().size() - 1);
  }

  private boolean isVisitedHashMap(Integer i) {
    Iterator<Integer> iter = visitedHashMap.keySet().iterator();

    while (iter.hasNext()) {
      Integer element = iter.next();
      if (visitedHashMap.get(element).contains(i)) {
        return true;
      }
    }
    return false;
  }

  /**
   * Returns this graph
   * 
   * @return this graph
   */
  public BidirectionalGraph getGraph() {
    return graph;
  }

  /**
   * sets a graph to be this graph
   * 
   * @param graph
   */
  public void setGraph(BidirectionalGraph graph) {
    this.graph = graph;
  }

  /**
   * Returns the clustering coefficient.
   */
  public double getClusteringCoefficient() {
    return clusteringCoefficient;
  }

  /**
   * Returns the link density
   */
  public BigDecimal getLinkDensity() {
    MathContext mc = new MathContext(32);
    BigDecimal edgesCount = new BigDecimal(getEdgesCount(), mc);
    BigDecimal setOfNodes = new BigDecimal(graph.getSetOfNodes().size(), mc);
    // BigDecimal result=new BigDecimal(graph.getSetOfNodes().size());
    BigDecimal divisor = ((setOfNodes.subtract(new BigDecimal(1))).multiply(setOfNodes)).divide(new BigDecimal(2), mc);
    return edgesCount.divide(divisor, mc);
  }

  /**
   * Gets the Edges Count
   */
  public Integer getEdgesCount() {
    return graph.getConnections().size();
  }

  /**
   * sets the clustering coefficient
   * 
   * @param clusteringCoefficient
   */
  public void setClusteringCoefficient(double clusteringCoefficient) {
    this.clusteringCoefficient = clusteringCoefficient;
  }

  /**
   * Returns the average path length
   */
  public double getAveragePathLength() {
    return averagePathLength;
  }

  /**
   * Sets the average Path length
   * 
   * @param averagePathLength
   */
  public void setAveragePathLength(double averagePathLength) {
    this.averagePathLength = averagePathLength;
  }

  /**
   * Returns the diameter of a graph. A diameter is the shortest path between
   * most distant vertices
   */
  public int getDiameter() {
    return diameter;
  }

  /**
   * Returns the Vertice count.
   */
  public int getNodesCount() {
    return graph.getNodesCount();
  }

  public void printStatisticsSummary() {
    System.out.println("Average path length\t" + this.getAveragePathLength());
    System.out.println("Clustering coefficient\t" + this.getClusteringCoefficient());
    System.out.println("Diameter\t\t" + this.getDiameter());
    System.out.println("Vertices count\t\t" + this.getNodesCount());
    System.out.println("Edges Count\t\t" + this.getEdgesCount());
    System.out.println("Link density\t\t" + this.getLinkDensity());
    System.out.println("Average Degree <k>\t" + this.getAverageDegree());
    System.out.println("Average Degree <k^2>\t" + this.getAverageDegreeQuadrat());
    System.out.println("Average Degree <k>^2\t" + Math.pow(this.getAverageDegree(), 2));
  }

  public String getStatisticsSummaryAsStringRepresentation() {
    String result = "";
    result = result.concat("#Average path length\t" + this.getAveragePathLength());
    result = result.concat("\n#Clustering coefficient\t" + this.getClusteringCoefficient());
    result = result.concat("\n#Diameter\t\t" + this.getDiameter());
    result = result.concat("\n#Vertices count\t\t" + this.getNodesCount());
    result = result.concat("\n#Edges Count\t\t" + this.getEdgesCount());
    result = result.concat("\n#Link density\t\t" + this.getLinkDensity().toString());
    result = result.concat("\n#Average Degree <k>\t" + this.getAverageDegree());
    result = result.concat("\n#Average Degree <k^2>\t" + this.getAverageDegreeQuadrat());
    result = result.concat("\n#Average Degree <k>^2\t" + Math.pow(this.getAverageDegree(), 2));
    return result;
  }

  public int getCliqueCount() {
    return cliqueCount;
  }

  public void setCliqueCount(int cliqueCount) {
    this.cliqueCount = cliqueCount;
  }

  public double getAverageDegree() {
    return averageDegree;
  }

  public void setAverageDegree(double averageDegree) {
    this.averageDegree = averageDegree;
  }

  public double getAverageDegreeQuadrat() {
    return averageDegreeQuadrat;
  }

  public void setAverageDegreeQuadrat(double averageDegreeQuadrat) {
    this.averageDegreeQuadrat = averageDegreeQuadrat;
  }

  public Vector<BidirectionalGraph> getIndependentSets() {
    return independentSets;
  }

  public HashMap<Integer, TreeSet<Integer>> getVisitedHashMap() {
    return visitedHashMap;
  }

}
