package brain.gui;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.CubicCurve2D;

import brain.Module;

/**
 * Shows the normalized distribution of activities on a 2D matrix
 * 
 * @author Gregor Kaczor
 * @author Mathias Linkerhand
 * @version 2010-09-10
 */
public class ActivityDistributionMatrixPanel extends ViewPanel {
  ActivityDistributionMatrixPanel(String _title, Module instance, String name) {
    super(_title, instance, name);
  }

  /**
   * 
   */
  private static final long serialVersionUID       = -8741406332943868929L;
  private int[][]           allPoints;
  private int               dimX                   = 0;
  private int               oldWidth               = 0;
  private int               oldHeigth              = 0;
  private int               oldNeuronsNumber       = 0;
  private int               dimY;
  private Color[]           gradients;
  private long              activityDistribution[] = null;

  // private double lastTime = Brain.time.getValue();

  @Override
  public void paint(Graphics g1) {
    if (gradients == null) {
      Color[] sourceColors = new Color[4];
      sourceColors[0] = Color.black;
      sourceColors[1] = new Color(0, 0, 255, 255);
      sourceColors[2] = Color.red;
      sourceColors[3] = Color.yellow;
      gradients = ReceptiveFieldCollection.createMultiGradient(sourceColors, 256);
    }

    Graphics2D g = (Graphics2D) g1;
    g.setStroke(new BasicStroke(2f));
    g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    g.clearRect(0, 0, this.getWidth(), this.getHeight());
    // --- ------------------------- ---
    // --- obtain module information ---
    // --- ------------------------- ---
    final int size = module.getSize();

    // if (activityDistribution == null || lastTime > Brain.time.getValue())
    // {
    activityDistribution = new long[size];
    // }
    // lastTime = Brain.time.getValue();
    if (dimX == 0 || oldNeuronsNumber != size) {
      dimX = 0;
      dimY = size;
      while ((dimX < dimY) || (dimX * dimY != size)) {
        dimX++;
        dimY = (int) (size / dimX);
      }
    }

    // --- ------------------------------------------- ---
    // --- print vertices on a two dimensional lattice ---
    // --- ------------------------------------------- ---
    if (oldWidth != this.getWidth() || oldHeigth != this.getHeight() || oldNeuronsNumber != size) {
      oldWidth = this.getWidth();
      oldHeigth = this.getHeight();
      oldNeuronsNumber = size;
      int startX = 30;
      int startY = 10;
      int stepX = (this.getWidth() - 50) / (dimX == 1 ? 1 : dimX - 1) - 5;
      int stepY = (this.getHeight() - 40) / (dimY == 1 ? 1 : dimY - 1);

      allPoints = new int[size][2];
      // 
      int iSite = 0;
      for (int iY = 0; iY < dimY; iY++)
        for (int iX = 0; iX < dimX; iX++) {
          allPoints[iSite][0] = startX + iX * stepX;
          allPoints[iSite][1] = startY + iY * stepY;
          iSite++;
        }

    }

    if (module.getSize() > 0) {
      // --- print associations as splines ---
      double arcValue = 0.10; // a measure for the curviness
      assert ((arcValue > 0.01) && (arcValue < 0.6));
      //
      for (int i = 0; i < size; i++) {

        for (int j = 0; j < i; j++) {
          if (module.getLink(i, j) > 0) {
            int x1 = allPoints[i][0];
            int y1 = allPoints[i][1];
            int x2 = allPoints[j][0];
            int y2 = allPoints[j][1];
            double middleX = 0.5 * (x2 + x1);
            double middleY = 0.5 * (y2 + y1);
            double deltaX = 1.0 * (x2 - x1);
            double deltaY = 1.0 * (y2 - y1);
            double xBetween = middleX + arcValue * deltaY;
            double yBetween = middleY - arcValue * deltaX;

            CubicCurve2D cc = new CubicCurve2D.Double((double) x1, (double) y1, xBetween, yBetween, xBetween, yBetween, (double) x2, (double) y2);
            // FIXME static activities threshold!!!!
            if ((module.getActivity(i) > 0.8) && (module.getActivity(j) > 0.8))
              g.setColor(Color.red);
            else {
              g.setColor(Color.blue);
            }
            g.draw(cc);
          }
        }
      }
    }
    long maximumCountOfActivations = Long.MIN_VALUE;
    long minimumCountOfActivations = Long.MAX_VALUE;
    for (int i = 0; i < size; i++) {
      if (module.getActivity(i) > 0.8)
        activityDistribution[i]++;
      if (maximumCountOfActivations < activityDistribution[i]) {
        maximumCountOfActivations = activityDistribution[i];
      }
      if (minimumCountOfActivations > activityDistribution[i]) {
        minimumCountOfActivations = activityDistribution[i];
      }
    }
    for (int i = 0; i < size; i++) {
      int xx = allPoints[i][0];
      int yy = allPoints[i][1];
      // int index=;
      g.setColor(gradients[(int) (((double) activityDistribution[i] / (double) maximumCountOfActivations) * 255)]);
      g.fillOval(xx - 7, yy - 7, 14, 14);
    }

    double totalSteps = this.getHeight() - 40;

    int heigth1 = 1;

    int width1 = 25;
    int y = 10;
    int x = this.getWidth() - 32;

    g.setColor(Color.black);
    g.drawString(new Long(maximumCountOfActivations).toString(), x, y);
    y += 4;

    for (double i = 0; i < totalSteps; i++) {
      g.setColor(gradients[(int) ((1 - (i / totalSteps)) * 255)]);
      g.fillRect(x, y, width1, heigth1);
      y += heigth1;
    }
    y += g.getFontMetrics().getHeight() - 8;
    g.setColor(Color.black);
    g.drawString(new Long(minimumCountOfActivations).toString(), x + 1, y + 2);
  }
}
