package brain.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.imageio.ImageIO;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.ListSelectionModel;
import javax.swing.SpringLayout;
import javax.swing.WindowConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileNameExtensionFilter;

import brain.Brain;
import brain.Module;
import brain.gui.table.ObjectTable;
import brain.gui.table.ObjectTableModel;

/**
 * adding and removing a module should update all opened frames showing
 * internals of modules <br>
 * PatternInputModule and SC_Test do not implement backwardConnections <br>
 * setting values in fields with primitive types <br>
 * setting values in fields with complex types, mainly arrays <br>
 * forward connections<br>
 * try to use annotations in the module edit table <br>
 *
 * @author Mathias Linkerhand
 * @version 2013-01-26
 * @since 2012
 */
public class BrainGUI extends JFrame implements ActionListener, ListSelectionListener, MouseListener, WindowListener {
  private static final long        serialVersionUID       = 1L;

  /** instance of the class which runs the simulation */
  private Brain                    brain                  = null;
  private Thread                   brainThread            = null;

  private Map<Module, ModuleFrame> moduleFrames;

  private static final Logger      logger                 = Logger.getLogger(BrainGUI.class.getName());

  final private JFileChooser       fileChooser;
  private JToggleButton            playPauseButton;
  private JButton                  stopButton;
  private JButton                  removeModuleButton;
  private JButton                  openModulesButton;
  private JList                    list;
  private DefaultListModel         listModel;
  private JScrollPane              brainTablePanel;

  public static final String       ABOUT_ACTION           = "about";
  public static final String       ADD_MODULE_ACTION      = "add module";
  public static final String       CONNECT_MODULES_ACTION = "connect modules";
  public static final String       LOAD_BRAIN_ACTION      = "load brain";
  public static final String       NEW_BRAIN_ACTION       = "new brain";
  public static final String       REMOVE_MODULE_ACTION   = "remove module";

  public BrainGUI(final Brain brainV) {
    //this(Brain.createDefaultLinkerhandBrain());
    //this(Brain.createDefaultGrosBrain());
    this();
    this.setBrain(brainV);
  }

  public BrainGUI() {
    this.addWindowListener(this);
    //this.setIconImage(new ImageIcon(getClass().getClassLoader().getResource("lib/images/brain.ico")).getImage());
    try {
      Image img = ImageIO.read(new File(Brain.RESOURCES_DIRECTORY, "brain.png"));
      this.setIconImage(img);
    } catch (IOException e1) {
      e1.printStackTrace();
    }

    this.moduleFrames = new HashMap<Module, ModuleFrame>();

    this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    this.setTitle("Brain");

    this.initMenuBar();

    // file chooser
    final JFileChooser fileChooser = new JFileChooser();
    this.fileChooser = fileChooser;
    //fileChooser.setCurrentDirectory(new File("."));
    fileChooser.setCurrentDirectory(Brain.DEFAULT_DATA_OUTPUT_DIRECTORY);
    fileChooser.setAcceptAllFileFilterUsed(true);
    fileChooser.addChoosableFileFilter(new FileNameExtensionFilter("Serialized Java Objects", "ser"));
    fileChooser.addChoosableFileFilter(new FileNameExtensionFilter("XML", "xml"));

    // save button
    final JButton saveModulesButton = new JButton("save …");
    saveModulesButton.setActionCommand("save");
    saveModulesButton.addActionListener(this);

    // load button
    final JButton loadModulesButton = new JButton("load …");
    loadModulesButton.setActionCommand(LOAD_BRAIN_ACTION);
    loadModulesButton.addActionListener(this);

    // start/pause button
    final JToggleButton playPauseButton = new JToggleButton("▶ start");
    this.playPauseButton = playPauseButton;
    playPauseButton.setActionCommand("play/pause");
    playPauseButton.addActionListener(this);
    playPauseButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        final JToggleButton button = (JToggleButton) e.getSource();
        button.setText(button.isSelected() ? "▷ start" : "▶ start");
      }
    });

    // stop button
    final JButton stopButton = new JButton("□ stop");
    this.stopButton = stopButton;
    stopButton.setActionCommand("stop");
    stopButton.addActionListener(this);
    stopButton.setEnabled(false);

    // button panel
    final JPanel buttonPanel = new JPanel(new GridLayout());
    buttonPanel.add(loadModulesButton);
    buttonPanel.add(saveModulesButton);
    buttonPanel.add(playPauseButton);
    buttonPanel.add(stopButton);

    // list model
    final DefaultListModel listModel = new DefaultListModel();
    this.listModel = listModel;

    // list
    final JList list = new JList(listModel);
    this.list = list;
    list.addListSelectionListener(this);
    list.addMouseListener(this);
    list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    list.setLayoutOrientation(JList.VERTICAL);
    list.setCellRenderer(new ModuleListCellRenderer());

    // add button
    final JButton addModuleButton = new JButton("+ add");
    addModuleButton.setActionCommand(ADD_MODULE_ACTION);
    addModuleButton.addActionListener(this);

    // remove button
    final JButton removeModuleButton = new JButton("– remove");
    removeModuleButton.setActionCommand(REMOVE_MODULE_ACTION);
    this.removeModuleButton = removeModuleButton;
    removeModuleButton.addActionListener(this);
    removeModuleButton.setEnabled(false);

    // open button
    final JButton openModulesButton = new JButton("open");
    this.openModulesButton = openModulesButton;
    openModulesButton.addActionListener(this);
    openModulesButton.setEnabled(false);

    // connect button
    final JButton connectButton = new JButton("connect module");
    connectButton.setActionCommand(CONNECT_MODULES_ACTION);
    connectButton.addActionListener(this);

    // disconnect button
    final JButton disconnectButton = new JButton("disconnect module");
    disconnectButton.setActionCommand("disconnect module");
    disconnectButton.addActionListener(this);

    // button pane
    final JPanel buttonPane = new JPanel();
    buttonPane.setLayout(new GridLayout());
    buttonPane.add(addModuleButton);
    buttonPane.add(removeModuleButton);
    buttonPane.add(openModulesButton);
    buttonPane.add(connectButton);
    buttonPane.add(disconnectButton);

    // brain parameter panel
    this.brainTablePanel = new JScrollPane();
    this.brainTablePanel.setMinimumSize(new Dimension(0, 150));

    // module list panel
    final JComponent moduleListPanel = new JScrollPane(list);
    moduleListPanel.setMinimumSize(new Dimension(0, 150));

    // split plane
    final JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    splitPane.add(this.brainTablePanel);
    splitPane.add(moduleListPanel);
    splitPane.setMinimumSize(new Dimension(Math.max(this.brainTablePanel.getMinimumSize().width, moduleListPanel.getMinimumSize().width), this.brainTablePanel
        .getMinimumSize().height + moduleListPanel.getMinimumSize().height + 400));

    this.setLayout(new BorderLayout());
    this.getContentPane().add(buttonPanel, BorderLayout.NORTH);
    this.getContentPane().add(splitPane, BorderLayout.CENTER);
    this.getContentPane().add(buttonPane, BorderLayout.SOUTH);

    this.setMinimumSize(new Dimension(400, 600));
    this.pack();

    // Resize window if it is too big (bigger than the screen size).
    final Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    this.setSize((int) Math.min(this.getWidth(), screenSize.getWidth()), (int) Math.min(this.getHeight(), screenSize.getHeight()));
  }

  private void initMenuBar() {
    // menu bar
    JMenuBar menuBar;
    JMenu menu;
    JMenuItem menuItem;

    menuBar = new JMenuBar();
    this.setJMenuBar(menuBar);

    menu = new JMenu("File");
    menu.setMnemonic(KeyEvent.VK_F);
    menuBar.add(menu);

    menuItem = new JMenuItem("New Brain", KeyEvent.VK_N);
    menuItem.setActionCommand(NEW_BRAIN_ACTION);
    menuItem.addActionListener(this);
    menu.add(menuItem);

    menu = new JMenu("Help");
    menu.setMnemonic(KeyEvent.VK_H);
    menuBar.add(menu);

    menuItem = new JMenuItem("About …", KeyEvent.VK_A);
    menuItem.setActionCommand(ABOUT_ACTION);
    menuItem.addActionListener(this);
    menu.add(menuItem);
  }

  public static Set<Constructor<?>> getAvailableModuleConstructors() {
    final Set<Constructor<?>> constructors = new HashSet<Constructor<?>>();
    for (final Class<?> clazz : BrainGUI.getAvailableModules()) {
      constructors.addAll(Arrays.asList(clazz.getConstructors()));
    }
    return constructors;
  }

  public static Set<Method> getAvailableModuleMethods() {
    final Set<Method> methods = new HashSet<Method>();
    for (final Class<?> clazz : BrainGUI.getAvailableModules()) {
      methods.addAll(Arrays.asList(clazz.getMethods()));
    }
    return methods;
  }

  public static Set<Constructor<?>> filterConstructorsWithEditableTypes(final Iterable<Constructor<?>> constructors) {
    final Set<Constructor<?>> filteredConstructors = new HashSet<Constructor<?>>();
    for (final Constructor<?> constructor : constructors) {
      boolean construtorEditableByGUI = true;
      for (final Class<?> clazz : constructor.getParameterTypes()) {
        if (!ObjectTableModel.isTypeAccepted(clazz, ObjectTableModel.EDITABLE_TYPES)) {
          construtorEditableByGUI = false;
          break;
        }
      }
      if (construtorEditableByGUI) {
        filteredConstructors.add(constructor);
      }
    }
    return filteredConstructors;
  }

  public static Set<Method> filterMethodsWithEditableTypes(final Iterable<Method> methods) {
    final Set<Method> filteredConstructors = new HashSet<Method>();
    for (final Method method : methods) {
      boolean construtorEditableByGUI = true;
      for (final Class<?> clazz : method.getParameterTypes()) {
        if (!ObjectTableModel.isTypeAccepted(clazz, ObjectTableModel.EDITABLE_TYPES)) {
          construtorEditableByGUI = false;
          break;
        }
      }
      if (construtorEditableByGUI) {
        filteredConstructors.add(method);
      }
    }
    return filteredConstructors;
  }

  public static Set<Class<?>> getAvailableModules() {
    Set<Class<?>> modules = new HashSet<Class<?>>();
    final File directory = new File("bin" + System.getProperty("file.separator") + "brain");
    logger.log(Level.INFO, "Searching for cognitive systems …");
    for (File file : directory.listFiles()) {
      if (file.getName().endsWith(".class")) {
        try {
          URL url = file.toURI().toURL();
          ClassLoader classLoader = new URLClassLoader(new URL[] { url });
          Class<?> clas = classLoader.loadClass("brain." + file.getName().substring(0, file.getName().length() - 6));

          // class should be instantiable, so no interfaces or abstract classes allowed
          if (!clas.isInterface() && !Modifier.isAbstract(clas.getModifiers())) {
            // is class implementing the interface Module?
            if (Module.class.isAssignableFrom(clas)) {
              logger.log(Level.INFO, "cognitive system “" + clas.getCanonicalName() + "” found");
              modules.add(clas);
            }
          }
        } catch (ClassNotFoundException exception) {
          exception.printStackTrace();
        } catch (MalformedURLException exception) {
          exception.printStackTrace();
        }
      }
    }
    logger.log(Level.INFO, "Searching for cognitive systems completed.");
    return modules;
  }

  public Brain getBrain() {
    return this.brain;
  }

  /**
   * Sets the brain object of the current Frame. Can be null.
   * @param brain Brain. Can be null.
   */
  private void setBrain(final Brain brain) {
    /*
     * Clear old brain.
     */
    // Terminate simulation.
    if (this.brainThread != null) {
      this.brainThread.interrupt();
    }

    // Close windows and remove modules and brain.
    this.listModel.clear();
    for (final JFrame frame : this.moduleFrames.values()) {
      frame.dispose();
    }
    this.moduleFrames.clear();
    this.brainTablePanel.getViewport().removeAll();

    /*
     * Set new brain.
     */
    // set new brain
    this.brain = brain;

    if (brain != null) {
      // add brain modules
      for (final Module module : brain.getModules()) {
        this.listModel.addElement(module);
      }

      // show brain panel
      final ObjectTable brainTable = new ObjectTable(brain);
      brain.addChangeListener(brainTable);
      brainTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
      brainTable.getColumn(brainTable.getModel().getColumnName(0)).setPreferredWidth(150);
      brainTable.getColumn(brainTable.getColumnName(1)).setPreferredWidth(60);
      brainTable.getColumn(brainTable.getColumnName(2)).setPreferredWidth(50);
      this.brainTablePanel.getViewport().add(brainTable);
    }

    // update UI
    this.updateButtonsOnStop();
  }

  /**
   * Sets buttons state when start button is pressed.
   */
  private void updateButtonsOnStart() {
    this.stopButton.setEnabled(true);
  }

  /**
   * Sets buttons state when stop button is pressed.
   */
  private void updateButtonsOnStop() {
    this.playPauseButton.getModel().setSelected(false);
    this.stopButton.setEnabled(false);
  }

  /**
   * Sets buttons state when pause button is pressed.
   */
  private void updateButtonsOnPause() {
    this.stopButton.setEnabled(true);
  }

  protected void openModuleFrame(Module module) {
    final ModuleFrame moduleFrame;
    if (moduleFrames.containsKey(module)) {
      moduleFrame = moduleFrames.get(module);
    } else {
      moduleFrame = new ModuleFrame(this.getBrain(), module);
      moduleFrame.setLocation(DisplayManager.calculateCoordinates(moduleFrame));
      moduleFrames.put(module, moduleFrame);
    }
    moduleFrame.setVisible(true);

    // deiconify frame
    moduleFrame.setState(Frame.NORMAL);
  }

  public void addModule(final Module module) {
    this.getBrain().addModule(module);
    this.listModel.addElement(module);
  }

  public void removeModule(final Module module) {
    if (this.moduleFrames.containsKey(module)) {
      this.moduleFrames.get(module).dispose();
      this.moduleFrames.remove(module);
    }

    this.getBrain().removeModule(module);
    this.listModel.removeElement(module);
  }

  public void connectModule() {
    final JComboBox modules = new JComboBox(this.getBrain().getModules().toArray());
    modules.setRenderer(new ModuleListCellRenderer());

    final JList afferentModules = new JList(new DefaultListModel());
    afferentModules.setCellRenderer(new ModuleListCellRenderer());

    final JPanel message = new JPanel();
    message.setLayout(new SpringLayout());
    message.add(new JLabel("module"));
    message.add(modules);
    message.add(new JLabel("afferent module"));
    message.add(new JScrollPane(afferentModules));
    message.add(new JLabel("synaptic weights"));
    final JTextField defaultSynapticWeightsTextField = new JTextField("1.0");
    message.add(defaultSynapticWeightsTextField);
    SpringUtilities.makeCompactGrid(message, 3, 2, 0, 0, 5, 5);

    final JOptionPane optionPane = new JOptionPane(message, JOptionPane.QUESTION_MESSAGE, JOptionPane.OK_CANCEL_OPTION);
    final JDialog dialog = optionPane.createDialog(this, "Connect modules.");

    modules.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent event) {
        final DefaultListModel listModel = (DefaultListModel) afferentModules.getModel();
        listModel.clear();

        final Module module = (Module) modules.getSelectedItem();
        if (module != null) {
          for (final Module afferentModule : getBrain().getModules()) {
            if (module != afferentModule && !module.hasAfferentModule(afferentModule)) {
              listModel.addElement(afferentModule);
            }
          }
        }
        dialog.pack();
      }
    });

    dialog.setVisible(true);
    final Object result = optionPane.getValue();
    if (result != null && result.equals(JOptionPane.OK_OPTION)) {
      for (final Object value : afferentModules.getSelectedValues()) {
        final Module afferentModule = (Module) value;
        Module module = (Module) modules.getSelectedItem();
        logger.log(Level.INFO, "connect module " + afferentModule + " to module " + module);
        module.addAfferentModule(afferentModule, Double.parseDouble(defaultSynapticWeightsTextField.getText()));
      }
    } else if (result == null || (result.equals(JOptionPane.CANCEL_OPTION) || result.equals(JOptionPane.CLOSED_OPTION))) {
    }
  }

  public void disconnectModule() {
    final DefaultComboBoxModel model = new DefaultComboBoxModel();
    for (final Module module : this.getBrain().getModules()) {
      if (module.getAfferentModules().size() > 0) {
        model.addElement(module);
      }
    }
    if (model.getSize() == 0) {
      JOptionPane.showMessageDialog(this, "There are no connected modules.", "No connected modules", JOptionPane.ERROR_MESSAGE);
      return;
    }
    final JComboBox modules = new JComboBox(model);
    modules.setRenderer(new ModuleListCellRenderer());

    final JList afferentModules = new JList(new DefaultListModel());
    afferentModules.setCellRenderer(new ModuleListCellRenderer());

    final JPanel message = new JPanel();
    message.setLayout(new SpringLayout());
    message.add(new JLabel("module"));
    message.add(modules);
    message.add(new JLabel("afferent modules"));
    message.add(new JScrollPane(afferentModules));
    SpringUtilities.makeCompactGrid(message, 2, 2, 0, 0, 5, 5);

    final JOptionPane optionPane = new JOptionPane(message, JOptionPane.QUESTION_MESSAGE, JOptionPane.OK_CANCEL_OPTION);

    final JDialog dialog = optionPane.createDialog(this, "disconnect modules");

    modules.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent event) {
        final DefaultListModel listModel = (DefaultListModel) afferentModules.getModel();
        listModel.clear();

        final Module module = (Module) modules.getSelectedItem();
        if (module != null) {
          for (final Module afferentModule : module.getAfferentModules()) {
            listModel.addElement(afferentModule);
          }
        }
        dialog.pack();
      }
    });

    dialog.setVisible(true);
    final Object result = optionPane.getValue();
    if (result != null && result.equals(JOptionPane.OK_OPTION)) {
      for (final Object value : afferentModules.getSelectedValues()) {
        Module afferentModule = (Module) value;
        Module module = (Module) modules.getSelectedItem();
        logger.log(Level.INFO, "disconnect module " + afferentModule + " from module " + module);
        module.removeAfferentModule(afferentModule);
      }
    } else if (result == null || (result.equals(JOptionPane.CANCEL_OPTION) || result.equals(JOptionPane.CLOSED_OPTION))) {
    }
  }

  /**
   * Adds a module to the brain. The module can be chosen via GUI given a list off all possible Modules and its constructors using the method BrainGUI.getAvailableModuleConstructors().
   */
  public void addModule() {
    final AccessibleObjectPreparationPanel accessibleObjectPreparationPanel = new AccessibleObjectPreparationPanel(
        BrainGUI.filterConstructorsWithEditableTypes(BrainGUI.getAvailableModuleConstructors()));
    final JOptionPane optionPane = new JOptionPane(accessibleObjectPreparationPanel, JOptionPane.QUESTION_MESSAGE, JOptionPane.OK_CANCEL_OPTION);
    final JDialog dialog = optionPane.createDialog(this, "Add module");
    accessibleObjectPreparationPanel.addPropertyChangeListener(new PropertyChangeListener() {
      @Override
      public void propertyChange(PropertyChangeEvent evt) {
        dialog.pack();
      }
    });
    while (true) {
      dialog.setVisible(true);
      final Object result = optionPane.getValue();
      if (result != null && result.equals(JOptionPane.OK_OPTION)) {
        final Constructor<?> constructor = (Constructor<?>) accessibleObjectPreparationPanel.getAccessibleObject();
        try {
          final Module loadedModule = Brain.loadModule(constructor, accessibleObjectPreparationPanel.getParameters());
          this.addModule(loadedModule);
          break;
        } catch (IllegalArgumentException exception) {
          logger.log(Level.WARNING, exception.getMessage());
          JOptionPane.showMessageDialog(this, "The given parameters cannot be used to create the module.", "Wrong parameters", JOptionPane.ERROR_MESSAGE);
        }
      } else if (result == null || (result.equals(JOptionPane.CANCEL_OPTION) || result.equals(JOptionPane.CLOSED_OPTION))) {
        break;
      }
    }
  }

  @Override
  public void actionPerformed(final ActionEvent actionEvent) {
    if (actionEvent.getActionCommand().equals("save")) {
      final int option = fileChooser.showSaveDialog(this);
      switch (option) {
        case JFileChooser.APPROVE_OPTION:
          File file = fileChooser.getSelectedFile();
          if (fileChooser.getFileFilter() instanceof FileNameExtensionFilter) {
            final FileNameExtensionFilter fileNameExtensionFilter = (FileNameExtensionFilter) fileChooser.getFileFilter();
            boolean addExtension = true;
            for (final String extension : fileNameExtensionFilter.getExtensions()) {
              if (file.getName().toLowerCase().endsWith("." + extension.toLowerCase())) {
                addExtension = false;
                break;
              }
            }
            if (addExtension) {
              file = new File(file.getParentFile(), file.getName() + "." + fileNameExtensionFilter.getExtensions()[0]);
            }
          }
          this.getBrain().save(file);
          break;
        case JFileChooser.CANCEL_OPTION:
          break;
        case JFileChooser.ERROR_OPTION:
          break;
      }
    } else if (actionEvent.getActionCommand().equals(LOAD_BRAIN_ACTION)) {
      /*
       * Load a new brain from the given file and open a new window while keeping the window for the old brain.
       */
      final int option = fileChooser.showOpenDialog(this);
      switch (option) {
        case JFileChooser.APPROVE_OPTION:
          final File file = fileChooser.getSelectedFile();
          try {
            final Brain brain = Brain.load(file);
            this.setBrain(brain);
          } catch (IOException e) {
            e.printStackTrace();
            JOptionPane.showMessageDialog(this, "Could not load modules from file " + file.getAbsolutePath(), "load module error", JOptionPane.ERROR_MESSAGE);
          } catch (ClassNotFoundException e) {
            e.printStackTrace();
            JOptionPane.showMessageDialog(this, "Could not load modules from file " + file.getAbsolutePath(), "load module error", JOptionPane.ERROR_MESSAGE);
          }
          break;
        case JFileChooser.CANCEL_OPTION:
          break;
        case JFileChooser.ERROR_OPTION:
          break;
      }
    } else if (actionEvent.getActionCommand().equals("stop")) {
      brainThread.interrupt();
      updateButtonsOnStop();
    } else if (actionEvent.getActionCommand().equals("play/pause")) {
      if (!playPauseButton.getModel().isSelected()) {
        this.getBrain().suspend();
        updateButtonsOnPause();
      } else if (this.getBrain().isSuspended()) {
        this.getBrain().resume();
        updateButtonsOnStart();
      } else {
        updateButtonsOnStart();
        brainThread = new Thread(this.getBrain());
        brainThread.start();

        // reset buttons on simulation end
        final Thread resetButtonOnSimulationEndThread = new Thread() {
          @Override
          public void run() {
            try {
              brainThread.join();
              updateButtonsOnStop();
            } catch (InterruptedException exception) {
              exception.printStackTrace();
            }
          }
        };
        resetButtonOnSimulationEndThread.start();
      }
    } else if (actionEvent.getActionCommand().equals(ADD_MODULE_ACTION)) {
      this.addModule();
    } else if (actionEvent.getActionCommand().equals(REMOVE_MODULE_ACTION)) {
      int index;
      while ((index = list.getSelectedIndex()) != -1) {
        this.removeModule((Module) this.listModel.get(index));
      }
    } else if (actionEvent.getActionCommand().equals("open")) {
      for (final int index : list.getSelectedIndices()) {
        this.openModuleFrame((Module) this.listModel.get(index));
      }
    } else if (actionEvent.getActionCommand().equals(CONNECT_MODULES_ACTION)) {
      this.connectModule();
    } else if (actionEvent.getActionCommand().equals("disconnect module")) {
      this.disconnectModule();
    } else if (actionEvent.getActionCommand().equals(NEW_BRAIN_ACTION)) {
      final Brain brain = new Brain();
      brain.setSimulationSleepDelay(100);
      this.setBrain(brain);
    } else if (actionEvent.getActionCommand().equals(ABOUT_ACTION)) {
      // about dialog
      ImageIcon icon = null;
      try {
        icon = new ImageIcon(new File(Brain.RESOURCES_DIRECTORY, "brain.png").toURI().toURL());
      } catch (HeadlessException e) {
        e.printStackTrace();
      } catch (MalformedURLException e) {
        e.printStackTrace();
      }
      final JLabel label = new JLabel("<html><h1>" + this.getTitle() + "</h1>" + "<p>"
          + "Frankfurt Cognitive System Online Monitoring<br/>by Claudius Gros, Gregor Kaczor, Mathias Linkerhand</p></html>");
      final String title = "About Brain";
      JOptionPane.showMessageDialog(this, label, title, JOptionPane.INFORMATION_MESSAGE, icon);
    }
  }

  @Override
  public void valueChanged(ListSelectionEvent e) {
    if (!e.getValueIsAdjusting()) {
      if (list.getSelectedIndex() == -1) {
        // no selection
        this.removeModuleButton.setEnabled(false);
        this.openModulesButton.setEnabled(false);
      } else {
        this.removeModuleButton.setEnabled(true);
        this.openModulesButton.setEnabled(true);
      }
    }
  }

  @Override
  public void mouseClicked(final MouseEvent mouseEvent) {
    if (mouseEvent.getClickCount() == 2) {
      final int index = list.locationToIndex(mouseEvent.getPoint());

      // list entry clicked?
      if (list.getCellBounds(index, index).contains(mouseEvent.getPoint())) {
        list.ensureIndexIsVisible(index);

        final Module module = (Module) list.getModel().getElementAt(index);
        this.openModuleFrame(module);
      }
    }
  }

  @Override
  public void mouseEntered(MouseEvent e) {
  }

  @Override
  public void mouseExited(MouseEvent e) {
  }

  @Override
  public void mousePressed(MouseEvent e) {
  }

  @Override
  public void mouseReleased(MouseEvent e) {
  }

  @Override
  public void windowActivated(WindowEvent e) {
  }

  @Override
  public void windowClosed(WindowEvent e) {
    for (final JFrame frame : this.moduleFrames.values()) {
      frame.dispose();
    }

    if (this.brainThread != null) {
      this.brainThread.interrupt();
    }
  }

  @Override
  public void windowClosing(WindowEvent e) {
  }

  @Override
  public void windowDeactivated(WindowEvent e) {
  }

  @Override
  public void windowDeiconified(WindowEvent e) {
  }

  @Override
  public void windowIconified(WindowEvent e) {
  }

  @Override
  public void windowOpened(WindowEvent e) {
  }
}