package brain.gui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.Window;

/**
 * Distributes components on screen in order to avoid overlapping. Uses a brute
 * force approach to find free space.
 * 
 * @author Gregor Kaczor
 * @author Mathias Linkerhand
 * @version 2010-09-21
 */

public class DisplayManager {
  /**
   * Calculates coordinates for a JFrame of a certain size. Tries to find a
   * not occupied rectangle on the screen.
   * 
   * @return coordinates of a not occupied rectangle of at least size d
   */
  public static Point calculateCoordinates(final Component component) {
    final Dimension dimension = component.getSize();
    final Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    final Rectangle screen = new Rectangle(screenSize);
    final Point result = new Point();
    final Rectangle nonIntersection = new Rectangle(0, 0, dimension.width, dimension.height);

    final int step = 1;
    for (int y = 0; y < screenSize.height; y += step) {
      for (int x = 0; x < screenSize.width; x += step) {
        nonIntersection.setLocation(x, y);
        if (!screen.contains(nonIntersection)) {
          // skip areas not on the screen
          continue;
        }
        boolean intersects = false;
        for (final Window window : Window.getWindows()) {
          if (window == component) {
            // skip window which should displayed later
            continue;
          }
          if (!window.isVisible()) {
            // skip unvisible windows
            continue;
          }
          final Rectangle rectangle = window.getBounds();

          if (rectangle.intersects(nonIntersection)) {
            x = window.getX() + window.getWidth() - 1;
            intersects = true;
            break;
          }
        }
        if (!intersects) {
          result.setLocation(x, y);
          return result;
        }
      }
    }
    return result;
  }
}