package brain.gui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;

import brain.Module;

/**
 * Displays a module/object which implements Module. This class does access
 * data via Reflection API. Sites are represented as elements of a matrix. Site
 * activities is shown by the color of every element. White is considered
 * inactive, black maximum activities. The color range is the gray color space.
 * The complete drawing scales with the size of the panel which it is drawn on.
 * 
 * @author Gregor Kaczor
 * @author Mathias Linkerhand
 * @version 2010-09-10
 */
public class MatrixViewPanel extends ViewPanel {

  /**
   * Instantiates MatrixViewPanel
   * 
   * @param _title
   * @param instance
   * @param name
   * 
   */
  public MatrixViewPanel(String _title, Module instance, String name) {
    super(_title, instance, name);
  }

  /**
   * serial id
   */
  private static final long serialVersionUID = 9091348126076117078L;
  /** width x axis*/
  private int               dimX;
  /** width y axis*/
  private int               dimY;
  /** number of neurons dimX and dimY have been calculated from*/
  private int               lastTotalNumberOfNeurons;

  /**
   * 
   * @see javax.swing.JComponent#paint(java.awt.Graphics)
   */
  @Override
  public void paint(Graphics g1) {
    Graphics2D g = (Graphics2D) g1;

    if (dimX == 0 || module.getSize() != lastTotalNumberOfNeurons) {
      dimX = 0;
      lastTotalNumberOfNeurons = module.getSize();
      while ((dimX < dimY) || (dimX * dimY != lastTotalNumberOfNeurons)) {
        dimX++;
        dimY = (int) (lastTotalNumberOfNeurons / dimX);
      }
    }

    int y = 0;
    int x = 0;
    int stepx = this.getWidth() / dimX;
    int stepy = this.getHeight() / dimY;
    for (int i = 0; i < lastTotalNumberOfNeurons; i++) {
      if (i % dimX == 0 && i > 0) {
        x = 0;
        y += stepy;
      }
      float color = 1 - (float) module.getActivity(i);
      if (color < 0)
        color = 0;
      if (color > 1)
        color = 1;
      g.setColor(new Color(color, color, color));
      g.fillRect(x, y, stepx, stepy);
      g.setColor(Color.black);
      g.drawRect(x, y, stepx, stepy);
      x += stepx;
    }
  }
}
