package brain.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;
import javax.swing.filechooser.FileNameExtensionFilter;

import utils.Utils;
import brain.Brain;
import brain.Module;
import brain.gui.table.ObjectTable;
import brain.gui.table.ObjectTableModel;

/**
 * @author Mathias Linkerhand
 * @version 2012-09-11
 * @since 2010
 */
public class ModuleFrame extends JFrame implements ActionListener, WindowListener {
  private static final long   serialVersionUID     = 1L;
  private Module              module;
  private ObjectTable         table;
  private File                parametersFile       = null;
  private JButton             reloadParameterButton;
  private List<Process>       processes            = new Vector<Process>();

  private static final Logger logger               = Logger.getLogger(ModuleFrame.class.getName());

  private List<JFrame>        childFrames          = new Vector<JFrame>();

  public static final String  ACTION_INVOKE_METHOD = "invoke method";

  public ModuleFrame(final Brain brain, final Module module) {
    this.module = module;

    this.addWindowListener(this);

    this.table = new ObjectTable(this.module);
    this.module.addChangeListener((ObjectTable) this.table);

    // load parameters button
    final JButton loadParametersButton = new JButton("load parameters …");
    loadParametersButton.setActionCommand("load parameters");
    loadParametersButton.addActionListener(this);

    // save parameters button
    final JButton saveParametersButton = new JButton("save parameters …");
    saveParametersButton.setActionCommand("save parameters");
    saveParametersButton.addActionListener(this);

    // reload parameters button
    final JButton reloadParameterButton = new JButton("reload parameters");
    this.reloadParameterButton = reloadParameterButton;
    reloadParameterButton.setEnabled(false);
    reloadParameterButton.setActionCommand("reload parameters");
    reloadParameterButton.addActionListener(this);

    // invoke method button
    final JButton invokeMethodButton = new JButton("invoke method");
    invokeMethodButton.setActionCommand(ACTION_INVOKE_METHOD);
    invokeMethodButton.addActionListener(this);

    // button panel
    final JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new GridLayout());
    buttonPanel.add(loadParametersButton);
    buttonPanel.add(saveParametersButton);
    buttonPanel.add(reloadParameterButton);
    buttonPanel.add(invokeMethodButton);

    /**
     * for every gnuplot file create a JButton which would start gnuplot and
     * load and display a gnuplot file
     */

    JButton button = new JButton("System View");
    button.setMaximumSize(new Dimension(Short.MAX_VALUE, button.getMaximumSize().height));
    button.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent event) {
        String name = ((JButton) event.getSource()).getText();
        JFrame frame = new SystemView(name + " " + module.getName(), module, "activities");
        childFrames.add(frame);
      }
    });
    buttonPanel.add(button);

    File[] files = Brain.DEFAULT_DATA_OUTPUT_DIRECTORY.listFiles(new FilenameFilter() {
      @Override
      public boolean accept(File dir, String name) {
        return name.endsWith(".gp") || name.endsWith(".gnuplot") || name.endsWith(".plt");
      }
    });

    this.setLayout(new BorderLayout());

    final boolean showStatisticsPanel = false;
    if (showStatisticsPanel) {
      // statistics panel
      final JPanel buttonStatisticsPanel = new JPanel();
      buttonStatisticsPanel.setLayout(new BoxLayout(buttonStatisticsPanel, BoxLayout.PAGE_AXIS));

      for (final File file : files) {
        button = new JButton(file.getName());
        button.setMaximumSize(new Dimension(Short.MAX_VALUE, button.getMaximumSize().height));
        button.addActionListener(new ActionListener() {
          @Override
          public void actionPerformed(ActionEvent event) {
            try {
              final String command = "gnuplot --persist " + file.getPath();
              logger.log(Level.INFO, "calling command " + command);
              final Process process = Runtime.getRuntime().exec(command);
              processes.add(process);
            } catch (IOException e1) {
              e1.printStackTrace();
            }
          }

        });

        buttonStatisticsPanel.add(button);
      }
      this.add(new JScrollPane(buttonStatisticsPanel), BorderLayout.EAST);
    }

    this.add(buttonPanel, BorderLayout.SOUTH);
    this.add(new JScrollPane(table), BorderLayout.CENTER);

    final String moduleName = this.module.getName();
    this.setTitle(moduleName);
    this.pack();
  }

  public Module getModule() {
    return this.module;
  }

  public void loadParameters() {
    final JFileChooser fileChooser = new JFileChooser();
    fileChooser.setSelectedFile(this.parametersFile);
    fileChooser.setAcceptAllFileFilterUsed(true);
    fileChooser.addChoosableFileFilter(new FileNameExtensionFilter("Module Parameter file", "par"));

    int result = fileChooser.showOpenDialog(this);
    if (result == JFileChooser.APPROVE_OPTION) {
      final File file = fileChooser.getSelectedFile();
      try {
        this.loadParameters(file);
        this.parametersFile = file;
      } catch (FileNotFoundException e1) {
        JOptionPane.showMessageDialog(this, "The file “" + file + "” was not found.", "File not found", JOptionPane.ERROR_MESSAGE);
      } catch (IOException e1) {
        JOptionPane.showMessageDialog(this, "Error reading file “" + file + "”.", "Read error", JOptionPane.ERROR_MESSAGE);
      }

      this.reloadParameterButton.setEnabled(true);
    }
  }

  public void reloadParameters() {
    final File file = this.parametersFile;
    try {
      this.loadParameters(file);
    } catch (FileNotFoundException e1) {
      JOptionPane.showMessageDialog(this, "The file “" + this.parametersFile + "” was not found.", "File not found", JOptionPane.ERROR_MESSAGE);
    } catch (IOException e1) {
      JOptionPane.showMessageDialog(this, "Error reading file “" + this.parametersFile + "”.", "Read error", JOptionPane.ERROR_MESSAGE);
    }
  }

  public void saveParameters() {
    final JFileChooser fileChooser = new JFileChooser();
    fileChooser.setSelectedFile(this.parametersFile);
    fileChooser.setAcceptAllFileFilterUsed(true);
    fileChooser.addChoosableFileFilter(new FileNameExtensionFilter("Module Parameter file", "par"));

    final int result = fileChooser.showSaveDialog(null);
    if (result == JFileChooser.APPROVE_OPTION) {
      final File file = fileChooser.getSelectedFile();
      try {
        this.saveParameters(file);
        this.parametersFile = file;
      } catch (FileNotFoundException e1) {
        JOptionPane.showMessageDialog(this, "The file “" + file + "” was not found.", "File not found", JOptionPane.ERROR_MESSAGE);
      } catch (IOException e1) {
        JOptionPane.showMessageDialog(this, "Error writing file “" + file + "”.", "Write error", JOptionPane.ERROR_MESSAGE);
      }
    }
  }

  public void invokeMethod() {
    final Set<Method> methods = Utils.getDeclaredMethods(this.getModule().getClass());

    final Set<Method> getterAndSetterMethods = new HashSet<Method>();
    for (final Method method : methods) {
      if (method.getName().startsWith("set") || method.getName().startsWith("get")) {
        getterAndSetterMethods.add(method);
      }
    }
    final SortedSet<Method> sortedMethods = new TreeSet<Method>(new Comparator<Method>() {
      @Override
      public int compare(Method o1, Method o2) {
        return o1.getName().compareTo(o2.getName());
      }
    });
    sortedMethods.addAll(BrainGUI.filterMethodsWithEditableTypes(getterAndSetterMethods));
    final AccessibleObjectPreparationPanel accessibleObjectPreparationPanel = new AccessibleObjectPreparationPanel(sortedMethods);
    final JOptionPane optionPane = new JOptionPane(accessibleObjectPreparationPanel, JOptionPane.QUESTION_MESSAGE, JOptionPane.OK_CANCEL_OPTION);
    final JDialog dialog = optionPane.createDialog(this, "Invoke method");
    accessibleObjectPreparationPanel.addPropertyChangeListener(new PropertyChangeListener() {
      @Override
      public void propertyChange(PropertyChangeEvent evt) {
        dialog.pack();
      }
    });
    while (true) {
      dialog.setVisible(true);
      final Object result = optionPane.getValue();
      if (result != null && result.equals(JOptionPane.OK_OPTION)) {
        final Method method = (Method) accessibleObjectPreparationPanel.getAccessibleObject();
        try {
          method.setAccessible(true);
          final Object methodResult = method.invoke(this.getModule(), accessibleObjectPreparationPanel.getParameters());
          this.table.updateUI();
          final String message = Arrays.deepToString(new Object[] { methodResult });
          logger.log(Level.INFO, "method returned: " + message);
          final JTextArea textArea = new JTextArea(20, 80);
          textArea.setText(message);
          textArea.setEditable(false);
          textArea.setLineWrap(true);
          textArea.setWrapStyleWord(true);
          final JScrollPane scrollPane = new JScrollPane(textArea);
          scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
          scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
          JOptionPane.showMessageDialog(this, scrollPane, "return value", JOptionPane.INFORMATION_MESSAGE);
          break;
        } catch (IllegalAccessException exception) {
          exception.printStackTrace();
        } catch (InvocationTargetException exception) {
          exception.printStackTrace();
        }
      } else if (result == null || (result.equals(JOptionPane.CANCEL_OPTION) || result.equals(JOptionPane.CLOSED_OPTION))) {
        break;
      }
    }
  }

  @Override
  public void actionPerformed(final ActionEvent actionEvent) {
    if (actionEvent.getActionCommand().equals("load parameters")) {
      this.loadParameters();
    } else if (actionEvent.getActionCommand().equals("reload parameters")) {
      this.reloadParameters();
    } else if (actionEvent.getActionCommand().equals("save parameters")) {
      this.reloadParameters();
    } else if (actionEvent.getActionCommand().equals(ACTION_INVOKE_METHOD)) {
      this.invokeMethod();
    }
  }

  /**
   * Loads the parameters for the current module from the given file. 
   * 
   * @param file
   * @throws FileNotFoundException
   * @throws IOException
   */
  public void loadParameters(final File file) throws FileNotFoundException, IOException {
    final ObjectInputStream in = new ObjectInputStream(new FileInputStream(file));

    try {
      while (true) {
        // read value
        final String name = (String) in.readObject();
        final Object value = in.readObject();

        // set value
        try {
          final Field field = Utils.getField(this.getModule().getClass(), name);
          field.setAccessible(true);
          field.set(this.getModule(), value);
          logger.log(Level.FINER, "field " + name + " ok");
        } catch (NoSuchFieldException e) {
          logger.log(Level.FINER, "field " + name + " not found");
        }
      }
    } catch (EOFException exception) {
      // end of file reached
    } catch (IOException e) {
      e.printStackTrace();
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
    } catch (IllegalArgumentException e) {
      e.printStackTrace();
    } catch (SecurityException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    }

    in.close();

    this.table.updateUI();
  }

  /**
   * Saves the parameters of the current module to the given file.
   * 
   * @param file
   * @throws FileNotFoundException
   * @throws IOException
   */
  public void saveParameters(final File file) throws FileNotFoundException, IOException {
    final ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(file));

    // Save all fields editable by the table.
    for (final AccessibleObject accessibleObject : ObjectTableModel.filterAccessibleObjects(this.getModule())) {
      if (accessibleObject instanceof Field) {
        final Field field = (Field) accessibleObject;
        if (ObjectTableModel.isTypeAccepted(field.getGenericType(), ObjectTableModel.EDITABLE_TYPES)) {
          // write field name
          out.writeObject(field.getName());
          try {
            // write field value
            field.setAccessible(true);
            out.writeObject(field.get(this.getModule()));
            logger.log(Level.FINER, "field " + field.getName() + " ok");
          } catch (IllegalArgumentException e) {
            logger.log(Level.FINER, "field " + field.getName() + " not ok");
          } catch (IllegalAccessException e) {
            logger.log(Level.FINER, "field " + field.getName() + " not ok");
          }
        } else {
          logger.log(Level.FINER, "field " + field.getName() + " not ok");
        }
      }
    }

    out.close();
  }

  @Override
  public void windowActivated(WindowEvent e) {
  }

  @Override
  public void windowClosed(WindowEvent e) {
    // close charts
    if (this.table.chartFrame != null) {
      //logger.log(Level.FINER, "dispose chart frame " + this.table.chartFrame);
      this.table.chartFrame.setVisible(false);
      this.table.chartFrame.dispose();
    }

    // close child frames
    for (final JFrame frame : childFrames) {
      logger.log(Level.FINER, "dispose child frame " + frame);
      frame.dispose();
    }

    // close gnuplot
    for (final Process process : processes) {
      logger.log(Level.FINEST, "end process " + process);
      process.destroy(); // This is not really working. Why?
    }
  }

  @Override
  public void windowClosing(WindowEvent e) {
  }

  @Override
  public void windowDeactivated(WindowEvent e) {
  }

  @Override
  public void windowDeiconified(WindowEvent e) {
  }

  @Override
  public void windowIconified(WindowEvent e) {
  }

  @Override
  public void windowOpened(WindowEvent e) {
  }
}