package brain.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Font;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;

import brain.Module;

/**
 * ModuleListCellRenderer renders modules used as list cells.
 * 
 * @author Mathias Linkerhand
 * @version 2013-01-08
 * @since 2011-04-05
 */
public class ModuleListCellRenderer extends DefaultListCellRenderer {
  private static final long serialVersionUID = 1L;
  private JPanel            rendererComponent;
  private JLabel            type             = new JLabel("");
  private JLabel            name             = new JLabel("");
  private JLabel            info             = new JLabel("");
  private JLabel            connections      = new JLabel("");

  public ModuleListCellRenderer() {
    this.type.setFont(this.type.getFont().deriveFont(Font.BOLD));
    this.info.setFont(this.info.getFont().deriveFont(Font.ITALIC));

    final JPanel rendererComponent = new JPanel(new BorderLayout(5, 0));
    this.rendererComponent = rendererComponent;
    rendererComponent.add(this.type, BorderLayout.NORTH);
    rendererComponent.add(this.name, BorderLayout.WEST);
    rendererComponent.add(this.info, BorderLayout.SOUTH);
    rendererComponent.add(this.connections, BorderLayout.CENTER);
    //rendererComponent.add(new JButton("click"), BorderLayout.EAST);
  }

  @Override
  public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
    final Module module = (Module) value;

    this.type.setText(module.getClass().getSimpleName());
    this.name.setText(module.getName());
    this.info.setText("N = " + module.getSize() + ", p = " + String.format("%.3f", module.getLinkProbability()) + ", degrees = "
        + module.getDegreeDistribution().toString().replace("=", ": "));

    String connectionString = null;
    for (Module afferentModule : module.getAfferentModules()) {
      if (connectionString == null) {
        connectionString = "";
      } else {
        connectionString += ", ";
      }
      connectionString += "← " + afferentModule.getName();
    }
    for (Module forwardModule : module.getForwardModules()) {
      if (connectionString == null) {
        connectionString = "";
      } else {
        connectionString += ", ";
      }
      connectionString += "→ " + forwardModule.getName();
    }
    this.connections.setText(connectionString);

    if (isSelected) {
      this.rendererComponent.setForeground(list.getSelectionForeground());
      this.rendererComponent.setBackground(list.getSelectionBackground());
    } else {
      this.rendererComponent.setForeground(list.getForeground());
      this.rendererComponent.setBackground(list.getBackground());
    }
    return this.rendererComponent;
  }
}