package brain.gui;

import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.JPanel;

/**
 * This class represents a panel on which receptive fileds are painted.
 * 
 * @author Gregor Kaczor
 * @version September 2008
 */
@SuppressWarnings("serial")
public class ReceptiveFieldPaintPanel extends JPanel {

  /** contains all receptive fields */
  private ReceptiveFieldCollection receptiveFieldCollecion;

  /**
   * Constructor instantiates ReceptiveFieldPaintPanel with a
   * ReceptiveFieldCollection.
   * 
   * @param _receptiveFieldCollecion
   */
  public ReceptiveFieldPaintPanel(final ReceptiveFieldCollection _receptiveFieldCollecion) {
    receptiveFieldCollecion = _receptiveFieldCollecion;
  }

  /**
   * Standard constructor
   */
  public ReceptiveFieldPaintPanel() {
  }

  /**
   * Paints receptive fields to the panel. Acually this method submits the paint
   * task to a method within ReceptiveFieldCollection.
   * 
   * @param g1 -
   *          a Graphics object to draw on.
   */
  @Override
  public void paint(final Graphics g1) {
    if (receptiveFieldCollecion != null) {
      receptiveFieldCollecion.setAbs_y(10);
      receptiveFieldCollecion.paint((Graphics2D) g1);
    }
  }

  /**
   * Returns ReceptiveFieldCollection
   * 
   * @return a ReceptiveFieldCollection
   */
  public ReceptiveFieldCollection getReceptiveFieldCollecion() {
    return receptiveFieldCollecion;
  }

  /**
   * Sets ReceptiveFieldCollection
   * 
   * @param receptiveFieldCollecion -
   *          a colletion of receptive fields
   */
  public void setReceptiveFieldCollecion(final ReceptiveFieldCollection receptiveFieldCollecion) {
    this.receptiveFieldCollecion = receptiveFieldCollecion;
  }
}
