package brain.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.JFrame;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import brain.Module;

/**
 * SystemView provides a tabbed pane and a notification mechanism implemented by
 * using observer pattern for repainting components drawn on tabs.
 * 
 * @author Gregor Kaczor
 * @author Mathias Linkerhand
 * @version 2011-01-10
 */
public class SystemView extends JFrame implements ChangeListener {
  private static final long serialVersionUID = 1L;
  private Module      module;
  private String      title;
  private JTabbedPane tabbedPane;
  private String      methodName;

  /**
   * Instantiates SystemView
   * 
   * @param _title -
   *          title of the frame which contains SystemView
   * @param _moduleInstance -
   *          instance of a module/class which implements Module
   * @param _methodName -
   *          name of a method which is passed to components displayed on tabs.
   *          This method is called to get a data when a notification event
   *          occures.
   */
  public SystemView(String _title, Module _moduleInstance, String _methodName) {
    title = _title;
    module = _moduleInstance;
    methodName = _methodName;

    this.setLayout(new BorderLayout());
    // frame.getRootPane().setDoubleBuffered(true);
    SystemViewPanel systemview = new SystemViewPanel(title, module, methodName);
    tabbedPane = new JTabbedPane();
    tabbedPane.addTab("Circle View", null, systemview, "Shows neurons with excitatory connections");

    MatrixViewPanel matrixview = new MatrixViewPanel(title, module, methodName);
    tabbedPane.addTab("Matrix View", null, matrixview, "Shows the activities of neurons in a gray scaled matrix. White corresponds to lack of activities.");

    ReceptiveFieldViewPanel receptiveFiledView = new ReceptiveFieldViewPanel(title, module, methodName);
    tabbedPane.addTab("Receptive Field View", null, receptiveFiledView, "Shows the receptive field in case this applies to a module");
    //add the chart to the frame:

    ActivityMatrixPanel linkedMatrixPanel = new ActivityMatrixPanel(title, module, methodName);
    tabbedPane.addTab("Activity Matrix View", null, linkedMatrixPanel, "Shows activities of cliques on a 2D matrix");

    tabbedPane.addTab("Activity Matrix View", null, linkedMatrixPanel, "Shows activities of cliques on a 2D matrix");

    ActivityDistributionMatrixPanel activityDistributionMatrixPanel = new ActivityDistributionMatrixPanel(title, module, methodName);
    tabbedPane.addTab("Activity Distribution View", null, activityDistributionMatrixPanel, "Shows normalized neuron activities distribution on a 2D matrix");

    LinkMatrixPanel colouredLinksMatrixPanel = new LinkMatrixPanel(title, module, methodName);
    tabbedPane.addTab("Link Matrix View", null, colouredLinksMatrixPanel, "Shows weigths of links and activities of neurons");

    //ReferencePatternViewPanel referencePatternViewPanel = new ReferencePatternViewPanel(title,
    //    module, methodName);
    //tabbedPane
    //    .addTab(
    //        "Reference PrintablePattern View",
    //        null,
    //        referencePatternViewPanel,
    //        "Shows refernce patterns");

    this.add(tabbedPane, BorderLayout.CENTER);
    this.setSize(new Dimension(400, 300));
    this.setLocation(DisplayManager.calculateCoordinates(this));

    this.setVisible(true);
    this.module.addChangeListener(this);
  }

  @Override
  public void stateChanged(ChangeEvent e) {
    this.repaint();
  }
}