package brain.gui;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;

import javax.swing.JPanel;

import brain.Module;

/**
 * Abstract class for painting different system views. Displays data from an
 * object implementing Module. Data is read by calling methods via Reflection
 * API.
 * 
 * @author Gregor Kaczor
 * @author Mathias Linkerhand
 * @version 2010-02-19
 */
public abstract class ViewPanel extends JPanel {
  /**
  	 * 
  	 */
  private static final long serialVersionUID = 1L;
  /** the object instance which implements Module */
  protected Module          module;
  /** method name to call to get data which is subject to display */
  protected String          methodName;
  /**
   * Field representing a variable of an object which value one might read via
   * <code>Field.get(Object o)</code>
   * 
   * @see java.lang.reflect.Field#get(Object)
   */
  protected Field           accessField;
  /**
   * Prefer access via <code>Field.get(Object o)</code> over calling a
   * getter-method
   */
  protected boolean         usePublicField;
  /** Title of the parent frame */
  protected String          title;

  /**
   * Standard constructor
   * 
   * @param _title
   *            - title of the panel
   * @param _moduleInstance
   *            - instance of a module we paint from. in case we need
   *            inter-modular data then one can access written simulation
   *            files and read them for further analysis.
   * @param _methodName
   *            - method we need to call via Reflection API to get data
   */
  ViewPanel(String _title, Module _moduleInstance, String _methodName) {
    module = _moduleInstance;
    title = _title;
    /**
     * Check whether we can use a filed access to get the value or if we
     * have to use a getter method
     */

    Field fields[] = module.getClass().getDeclaredFields();
    for (int i = 0; i < fields.length; i++) {
      if (fields[i].getName().compareTo(_methodName) == 0 && fields[i].getModifiers() == Modifier.PUBLIC) {
        usePublicField = true;
        accessField = fields[i];
        break;
      }
    }
    methodName = "get" + _methodName.substring(0, 1).toUpperCase() + _methodName.substring(1);
  }
}
