package brain.gui.chart;

import info.monitorenter.gui.chart.ZoomableChart;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.KeyStroke;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileNameExtensionFilter;

/**
 * @author Mathias Linkerhand
 * @version 2012-09-11
 * @since 2010
 */
public class ChartFrame extends JFrame implements WindowListener {
  private static final long serialVersionUID = 1L;

  protected Vector<Chart>   charts           = new Vector<Chart>();

  protected JPanel          graphPanel;

  public ChartFrame(final Chart chart) {
    this();
    this.addChart(chart);
  }

  public ChartFrame() {
    this.addWindowListener(this);
    this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    this.setTitle("chart");

    this.setSize(500, 600);

    final ChartFrame self = this;

    this.setLayout(new BorderLayout());

    // chart management panel
    final JPanel chartManagementPanel = new JPanel();
    chartManagementPanel.setLayout(new FlowLayout());

    final JLabel strokeJLabel = new JLabel("Stroke", SwingConstants.CENTER);
    SpinnerModel spinnerNumberModel = new SpinnerNumberModel(1, 1, 10, 1);

    final JSpinner strokeSpinner = new JSpinner(spinnerNumberModel);
    strokeSpinner.addChangeListener(new ChangeListener() {
      @Override
      public void stateChanged(ChangeEvent event) {
        final JSpinner spinner = (JSpinner) event.getSource();
        int strokeWidth = ((Integer) spinner.getValue()).intValue();
        for (Chart chart : charts) {
          chart.setStroke(strokeWidth);
        }
      }
    });

    chartManagementPanel.add(strokeJLabel);
    chartManagementPanel.add(strokeSpinner);

    JButton zoomOutButton = new JButton("Zoom Out");
    zoomOutButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        for (Chart chart : charts) {
          chart.zoomFull();
        }
      }
    });
    chartManagementPanel.add(zoomOutButton);

    zoomOutButton.getInputMap().put(KeyStroke.getKeyStroke("SPACE"), "pressed");

    final JButton exportButton = new JButton("Export...");
    exportButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        JFileChooser fileChooser = null;
        try {
          File f = new File(new File(".").getCanonicalPath());
          fileChooser = new JFileChooser();
          fileChooser.setCurrentDirectory(f);
          fileChooser.setAcceptAllFileFilterUsed(true);
          fileChooser.setDialogTitle("Export chart to file");
        } catch (IOException e2) {

          e2.printStackTrace();
          return;
        }

        String formats[] = ImageIO.getReaderFormatNames();
        for (int i = 0; i < formats.length; i++) {
          FileNameExtensionFilter fileChooserMultiFilter = new FileNameExtensionFilter(formats[i] + " image", formats[i]);
          fileChooser.addChoosableFileFilter(fileChooserMultiFilter);
        }

        // Open file dialog.
        int result = fileChooser.showSaveDialog(null);

        // Determine which button was clicked to close the dialog
        switch (result) {
          case JFileChooser.APPROVE_OPTION:
            try {
              export(new File(fileChooser.getSelectedFile().getAbsolutePath() + "." + fileChooser.getFileFilter().getDescription()), fileChooser
                  .getFileFilter().getDescription());
            } catch (IOException exception) {
              JOptionPane.showMessageDialog(self, exception.toString(), "error during save", JOptionPane.ERROR_MESSAGE);
            }
            break;
          case JFileChooser.CANCEL_OPTION:
            return;
          case JFileChooser.ERROR_OPTION:
            return;
        }
      }

    });
    chartManagementPanel.add(exportButton);

    // trace length
    final JLabel traceLengthLabel = new JLabel("traces");
    final SpinnerModel traceLengthSpinnerNumberModel = new SpinnerNumberModel(300, 100, 10000, 100);
    final JSpinner traceLengthSpinner = new JSpinner(traceLengthSpinnerNumberModel);
    traceLengthSpinner.addChangeListener(new ChangeListener() {
      @Override
      public void stateChanged(ChangeEvent event) {
        final JSpinner spinner = (JSpinner) event.getSource();
        final int traceLength = (Integer) spinner.getValue();
        for (Chart chart : charts) {
          chart.setTraceLength(traceLength);
        }
      }
    });
    chartManagementPanel.add(traceLengthLabel);
    chartManagementPanel.add(traceLengthSpinner);

    this.getContentPane().add(chartManagementPanel, BorderLayout.SOUTH);

    final JPanel graphPanel = new JPanel();
    graphPanel.setLayout(new BoxLayout(graphPanel, BoxLayout.Y_AXIS));
    this.add(graphPanel, BorderLayout.CENTER);
    this.graphPanel = graphPanel;
  }

  public void export(final File file, final String formatName) throws IOException {
    ChartFrame.exportComponent(this.charts.firstElement(), file, formatName);
  }

  public static void exportComponent(final Component component, final File file, final String formatName) throws IOException {
    Dimension size = component.getSize();
    BufferedImage myImage = new BufferedImage(size.width, size.height, BufferedImage.TYPE_INT_RGB);
    Graphics2D graphics2D = myImage.createGraphics();
    component.paint(graphics2D);
    OutputStream out = new FileOutputStream(file);
    ImageIO.write(myImage, formatName, out);
    out.close();
  }

  public void updateTitle() {
    final String title;
    if (this.charts.isEmpty()) {
      title = "charts";
    } else {
      title = this.charts.get(0).object.getClass().getSimpleName();
    }
    this.setTitle(title);
  }

  public Component getChart(int index) {
    return this.charts.get(index);
  }

  public Vector<Chart> getCharts() {
    return this.charts;
  }

  public void addChart(final Chart chart) {
    this.graphPanel.add(chart);
    this.graphPanel.validate();
    this.graphPanel.repaint();
    this.charts.add(chart);
    this.updateTitle();
  }

  public void removeChart(final Chart chart) {
    this.graphPanel.remove(chart);
    this.graphPanel.validate();
    this.graphPanel.repaint();
    this.charts.remove(chart);
    this.updateTitle();

    // Bug in JChart2D? However, ZoomableCharts have to be destroyed.
    if (chart.getChart() instanceof ZoomableChart) {
      ((ZoomableChart) chart.getChart()).destroy();
    }
  }

  @Override
  public void windowActivated(WindowEvent e) {
  }

  @Override
  public void windowClosed(WindowEvent e) {
    // Bug in JChart2D? However, ZoomableCharts have to be destroyed, there remove them.
    for (Chart chart : this.charts.toArray(new Chart[0])) {
      if (chart.getChart() instanceof ZoomableChart) {
        this.removeChart(chart);
      }
    }
  }

  @Override
  public void windowClosing(WindowEvent e) {
  }

  @Override
  public void windowDeactivated(WindowEvent e) {
  }

  @Override
  public void windowDeiconified(WindowEvent e) {
  }

  @Override
  public void windowIconified(WindowEvent e) {
  }

  @Override
  public void windowOpened(WindowEvent e) {
  }
}