package brain.gui.chart;

import info.monitorenter.gui.chart.ITrace2D;
import info.monitorenter.gui.chart.ZoomableChart;
import info.monitorenter.gui.chart.traces.Trace2DLtd;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.lang.reflect.Field;
import java.util.Vector;

/**
 * @author Mathias Linkerhand
 * @version 2011-09-21
 */
public class JChart2D extends Chart {
  private static final long serialVersionUID = 1L;

  private Vector<ITrace2D>  traces;

  public JChart2D(final Field fieldX, final Field fieldY, final Object object) {
    super(fieldX, fieldY, object);

    final ZoomableChart chart = new ZoomableChart();
    chart.setAutoscrolls(true);
    chart.setDoubleBuffered(true);
    chart.getAxisX().getAxisTitle().setTitle("t");
    chart.getAxisY().getAxisTitle().setTitle(fieldY.getName());

    this.setChart(chart);

    this.traces = new Vector<ITrace2D>();

    final Dimension size = new Dimension(500, 300);
    this.setPreferredSize(size);
  }

  @Override
  protected void addPoints(double time, double[] values) {
    for (int i = this.traces.size(); i < values.length; i++) {
      final ITrace2D trace = new Trace2DLtd(300);
      trace.setColor(Color.getHSBColor(1f / values.length * i, 1f, 0.8f)); // use rainbow colors for different graphs
      trace.setName(Integer.toString(this.traces.size(), Character.MAX_RADIX));
      this.traces.add(trace);
      ((ZoomableChart) this.getChart()).addTrace(trace);
    }
    for (int i = 0; i < values.length; i++) {
      this.traces.get(i).addPoint(time, values[i]);
    }
  }

  @Override
  public void setTraceLength(final int traceLength) {
    for (final ITrace2D trace : traces) {
      ((Trace2DLtd) trace).setMaxSize(traceLength);
    }
  }

  @Override
  public boolean isZoomable() {
    return true;
  }

  @Override
  public void zoomFull() {
    ((ZoomableChart) this.getChart()).zoomAll();
  }

  @Override
  public void setStroke(int stroke) {
    final BasicStroke basicStroke = new BasicStroke(stroke);
    for (ITrace2D trace : this.traces) {
      trace.setStroke(basicStroke);
    }
  }
}