package brain.gui.chart;

import java.awt.Toolkit;
import java.lang.reflect.Field;
import java.util.Date;

import javax.swing.TransferHandler;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * @author Mathias Linkerhand
 * @version 2011-09-21
 */
public class JFreeChart extends Chart {
  private static final long  serialVersionUID = 1L;

  private XYSeriesCollection traces;
  private long               lastRepaint;
  private long               repaintInterval  = 10;

  public JFreeChart(final Field fieldX, final Field fieldY, final Object object) {
    super(fieldX, fieldY, object);

    final String title = object.getClass().getSimpleName() + ", " + fieldY.getName();
    this.setSize(Toolkit.getDefaultToolkit().getScreenSize().width / 3, Toolkit.getDefaultToolkit().getScreenSize().height / 3);

    this.traces = new XYSeriesCollection();

    final ChartPanel chartPanel = new ChartPanel(ChartFactory.createXYLineChart(title /* title */, "time" /* domain */, "value" /* range */, this.traces,
        PlotOrientation.VERTICAL, true /* legend */, true /* tool tips */, false /* URLs */));
    this.setChart(chartPanel);

    // drag and drop
    chartPanel.setTransferHandler(new JFreeChart.FromTransferHandler());
  }

  @Override
  synchronized protected void addPoints(double time, double[] values) {
    // create new traces
    while (this.traces.getSeries().size() < values.length) {
      final XYSeries trace = new XYSeries(Integer.toString(this.traces.getSeries().size(), Character.MAX_RADIX));
      trace.setMaximumItemCount(100);
      final XYDataset data = new XYSeriesCollection();
      final XYPlot plot = new XYPlot();
      plot.setDataset(data);
      this.traces.addSeries(trace);
    }

    // add points to traces
    for (int i = 0; i < values.length; i++) {
      //final boolean repaint = i == values.length - 1; // Repaint if and only if the last trace is updated.
      final boolean repaint = false; // Repaint will be handled later.
      ((XYSeries) this.traces.getSeries().get(i)).add(time, values[i], repaint);
    }

    // repaint
    final long now = new Date().getTime();
    if (this.lastRepaint < new Date().getTime() - this.repaintInterval) {
      this.lastRepaint = now;
      ((XYSeries) this.traces.getSeries().get(values.length - 1)).fireSeriesChanged();
    }
  }

  @Override
  public boolean isZoomable() {
    return false;
  }

  @Override
  public void setTraceLength(int traceLength) {
    for (Object trace : this.traces.getSeries()) {
      ((XYSeries) trace).setMaximumItemCount(traceLength);
    }
  }

  @Override
  public void setStroke(int stroke) {
  }

  @Override
  public void zoomFull() {
  }

  class FromTransferHandler extends TransferHandler {
    private static final long serialVersionUID = 1L;
  }
}