package brain.gui.table;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Field;
import java.lang.reflect.Type;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JTable;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import utils.Utils;
import brain.Module;
import brain.gui.DisplayManager;
import brain.gui.chart.Chart;
import brain.gui.chart.ChartFrame;
import brain.gui.chart.JChart2D;
import brain.gui.chart.JFreeChart;

/**
 * @author Mathias Linkerhand
 * @version 2012-09-11
 * @since 2010
 */
public class ObjectTable extends JTable implements ChangeListener {
  private static final long serialVersionUID = 1L;

  private Map<Field, Chart> charts           = new HashMap<Field, Chart>();

  public ChartFrame         chartFrame;
  private double            printInterval    = 0;
  private Date              lastMessage      = new Date();
  private Object            object;
  private ObjectTableModel  objectTableModel;

  public ObjectTable(final Object object) {
    super();
    this.object = object;
    this.objectTableModel = new ObjectTableModel(object);
    this.setModel(objectTableModel);

    // cell renderers
    this.setDefaultRenderer(Type.class, new TypeTableCellRenderer());
    this.setDefaultRenderer(AccessibleObject.class, new AccessibleObjectTableCellRenderer());
    this.setDefaultRenderer(Boolean.class, new BooleanTableCellRenderer());

    // cell editors
    this.setDefaultEditor(AccessibleObject.class, new AccessibleObjectTableCellEditor());
    this.setDefaultEditor(Boolean.class, new BooleanTableCellEditor());

    this.getModel().addTableModelListener(new TableModelListener() {
      @Override
      public void tableChanged(final TableModelEvent event) {
        if (event.getType() == TableModelEvent.UPDATE && event.getFirstRow() != -1 && event.getColumn() == 3) {
          TableModel model = (TableModel) event.getSource();
          final boolean isSelected = (Boolean) model.getValueAt(event.getFirstRow(), 3);
          final Field field = (Field) model.getValueAt(event.getFirstRow(), 0);

          if (isSelected) {
            if (!charts.containsKey(field)) {
              final int chartType = 0;
              try {
                final Chart chart;
                switch (chartType) {
                  case 0:
                  default:
                    chart = new JFreeChart(Utils.getField(Module.class, "elapsedTime"), field, object);
                    break;
                  case 1:
                    chart = new JChart2D(Utils.getField(Module.class, "elapsedTime"), field, object);
                    break;
                }
                if (chartFrame == null) {
                  chartFrame = new ChartFrame();
                  chartFrame.setLocation(DisplayManager.calculateCoordinates(chartFrame));
                }
                chartFrame.addChart(chart);
                chartFrame.setVisible(true);

                charts.put(field, chart);
              } catch (NoSuchFieldException e1) {
                e1.printStackTrace();
              }
            } else {
              charts.get(field).setVisible(true);
            }
          } else {
            if (charts.containsKey(field)) {
              //charts.get(field).setVisible(false);
              chartFrame.removeChart(charts.get(field));
              if (chartFrame.getCharts().size() == 0) {
                chartFrame.dispose();
              }
              charts.remove(field);
            }
          }
        }
      }
    });

    //    this.getColumnModel().getColumn(0).setPreferredWidth(170);
    //    this.getColumnModel().getColumn(1).setPreferredWidth(50);
    //    this.getColumnModel().getColumn(2).setPreferredWidth(50);
    this.getColumnModel().getColumn(ObjectTableModel.OBSERVE_COLUMN).setMaxWidth(65);
  }

  @Override
  public void stateChanged(ChangeEvent e) {
    if (e.getSource() == this.object) {
      if (new Date().getTime() - this.lastMessage.getTime() > this.printInterval) {
        this.lastMessage = new Date();
        
        // Update table with new values.
        this.objectTableModel.fireTableDataChanged();

        if (this.chartFrame != null) {
          for (final Chart chart : this.chartFrame.getCharts()) {
            // Update charts with new values.
            chart.stateChanged(e);
          }
        }
      }
    }
  }
}