package brain.gui.table;

import java.awt.Component;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

/**
 * @author Mathias Linkerhand
 * @version 2011-02-16
 */
public class TypeTableCellRenderer extends DefaultTableCellRenderer {
  private static final long serialVersionUID = 1L;

  @Override
  public Component getTableCellRendererComponent(final JTable table, final Object value, final boolean isSelected, final boolean hasFocus, final int row,
      final int column) {
    final Component component = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
    if (value instanceof Type) {
      final Type type = (Type) value;
      final JLabel label = (JLabel) component;
      label.setText(TypeTableCellRenderer.classToString(type));
      label.setToolTipText(type.toString());
    }else{
      //System.out.println(this.getClass().getName() + " should not be used for type " + value.getClass().getName());
    }
    return component;
  }

  public static String classToString(final Type type) {
    String result = "";

    if (type instanceof ParameterizedType) {
      final ParameterizedType parameterizedType = (ParameterizedType) type;
      result += ((Class<?>) parameterizedType.getRawType()).getSimpleName();
      result += "<";
      boolean isFirst = true;
      for (final Type typeArgument : parameterizedType.getActualTypeArguments()) {
        if (isFirst) {
          isFirst = false;
        } else {
          result += ", ";
        }
        result += classToString(typeArgument);
      }
      result += ">";
    } else {
      result += ((Class<?>) type).getSimpleName();
    }
    return result;
  }
}
