package calculation;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Random;
import java.util.Set;
import java.util.Vector;

import brain.LeakyIntegratorWithIntrinsicPlasticityModule;

import utils.Utils;

public class CliqueLinksScaling {

  public static void main(String[] args) {
    double r_min_last = 0;
    probe: for (int graphSize = 10; graphSize < 100; graphSize += 10) {
      final double linkProbability = 0.3;
      final Random random = new Random(0);
      LeakyIntegratorWithIntrinsicPlasticityModule module = new LeakyIntegratorWithIntrinsicPlasticityModule(graphSize, linkProbability, random);
      module.setIntrinsicPlasticity(false);
      module.setTransferFunctionParameters(4.0, 1.0);
      module.normalizeLinks();
      module.getCliques(); // precalculate cliques
      System.out.println("N=" + graphSize + ", cliques=" + module.getCliques().size());

      module.setName("scaling test");
      // save module for later restore after manipulations/changes
      final byte[] serializedModule = Utils.objectToBytes(module);

      // Sort cliques by size in order to test big cliques first.
      final List<Set<Integer>> cliques = new Vector<Set<Integer>>(module.getCliques());
      Collections.sort(cliques, new Comparator<Set<Integer>>() {
        @Override
        public int compare(Set<Integer> o1, Set<Integer> o2) {
          return o1.size() < o2.size() ? 1 : (o1.size() > o2.size() ? -1 : 0);
        }
      });
      System.out.println(cliques);

      for (double r = r_min_last; r < 10; r += 0.3) {
        final double Δφ = Math.PI / 2 / (r + 1);
        final double φ_min = Math.PI / 2 + Δφ;
        final double φ_max = Math.PI;
        for (double φ = φ_min; φ < φ_max; φ += Δφ) {
          final double w = r * Math.sin(φ);
          final double z = r * Math.cos(φ);
          System.out.println("r=" + r + ", φ=" + Math.toDegrees(φ) + "°");
          System.out.println("probing (N, w, z) = (" + graphSize + ", " + w + ", " + z + ")");

          // restore module
          module = (LeakyIntegratorWithIntrinsicPlasticityModule) Utils.bytesToObject(serializedModule);

          // set new parameters w and z
          for (int i = 0; i < module.getSize(); i++) {
            for (int j = 0; j < module.getSize(); j++) {
              final double oldWeight = module.getLink(i, j);
              final double newWeight;
              if (oldWeight < 0) {
                // inhibitory link
                newWeight = -z * oldWeight;
              } else if (oldWeight > 0) {
                // exhibitory link
                newWeight = w * oldWeight;
              } else {
                // no link
                newWeight = 0;
              }
              module.setLink(i, j, newWeight);
            }
          }

          int cliqueNumber = 0;
          for (Set<Integer> clique : cliques) {
            cliqueNumber++;
            System.out.print(cliqueNumber + " ");

            // activate neurons of one clique, deactivate others
            for (int neuron = 0; neuron < module.getSize(); neuron++) {
              if (clique.contains(neuron)) {
                module.setActivity(neuron, 0.9);
              } else {
                module.setActivity(neuron, 0.1);
              }
            }

            // run simulation
            final double Δt = 1e-3;
            final double t_max = 1e3;
            for (double t = 0; t <= t_max; t += Δt) {
              module.performOneTimeStep(Δt);
            }

            // check for active clique
            if (module.getActiveClique() != null) {
              System.out.println();
              System.out.println("size=" + graphSize + ", w=" + w + ", z=" + z + ", clique=" + module.getActiveClique());
              System.out.println();
              r_min_last = r;
              continue probe;
            }

            if (cliqueNumber == module.getCliques().size()) {
              System.out.println();
            }
          }
        }
      }
      System.out.println("No stable clique found in given phase space region for N=" + graphSize + "!");
      System.out.println();
    }
  }
}
