package calculation;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.Locale;
import java.util.Random;

import utils.Graphs;

import brain.LeakyIntegratorWithIntrinsicPlasticityModule;
import brain.Brain;
import brain.Module;

/**
 * Simulates a single neuron and its intrinsic plasticity.
 * 
 * @author Mathias Linkerhand
 * @version 2012-02-03
 */
public class NetworkIntrinsicPlasticity {

  public static void main(String[] args) throws FileNotFoundException {
    // what about constant input?
    final Brain brain = new Brain();
    final double timeDelta = 1e-2;
    final double timeMax = 1e3;
    brain.setTimeDeltaAndDuration(timeDelta, timeMax);
    brain.setLogInterval(Math.max(timeDelta, timeMax / 1e4)); // 1e5 data points

    final Random random = new Random(0);
    //final int size = 10;
    //final double linkProbability = 0.4;
    //final LeakyIntegratorWithIntrinsicPlasticityModule relisi = new LeakyIntegratorWithIntrinsicPlasticityModule(size, linkProbability, random);
    final double[][] adjacencyMatrix = Graphs.addInhibitoryLinks(Graphs.grosGraphAdjacencyMatrix);
    final LeakyIntegratorWithIntrinsicPlasticityModule relisi = new LeakyIntegratorWithIntrinsicPlasticityModule(adjacencyMatrix, random);
    relisi.setName("Relisi");
    relisi.setLearning(true);
    relisi.setLearningRates(1e-1);
    relisi.setLearningType(Module.OJAS_RULE_LEARNING);

    System.out.println(relisi.getLinks());

    PrintWriter out = new PrintWriter(new File(brain.dataOutputDirectory, "links.data"));
    for (int i = 0; i < relisi.getSize(); i++) {
      for (int j = 0; j < relisi.getSize(); j++) {
        out.printf(Locale.ENGLISH, "%d\t%d\t%f\n", i, j, relisi.getLink(i, j));
      }
      out.println();
    }
    out.close();

    out = new PrintWriter(new File(brain.dataOutputDirectory, "link-matrix.data"));
    for (int i = 0; i < relisi.getSize(); i++) {
      for (int j = 0; j < relisi.getSize(); j++) {
        if (j != 0) {
          out.print("\t");
        }
        out.printf(Locale.ENGLISH, "%f", relisi.getLink(i, j));
      }
      out.println();
    }
    out.close();

    // intrinsic learning rate
    final double intrinsicAdaptationParameter = 1e-2;
    for (int neuron = 0; neuron < relisi.getSize(); neuron++) {
      final double currentIntrinsicAdaptationParameter = intrinsicAdaptationParameter;
      relisi.setIntrinsicAdaptationLearningRates(neuron, currentIntrinsicAdaptationParameter);
    }

    // leaks
    final double leak = 1e0;
    for (int neuron = 0; neuron < relisi.getSize(); neuron++) {
      final double currentLeak = leak;
      relisi.setLeak(neuron, currentLeak);
    }

    // default transfer function parameters
    relisi.setTransferFunctionParameters(1.0, 1.0); // gain, threshold

    // default membrane potential
    for (int neuron = 0; neuron < relisi.getSize(); neuron++) {
      //      relisi.setMembranePotential(neuron, 0.1);
    }

    // target distributions
    relisi.setTargetDistributionParameters(-10.0, 0.0);

    // logger properties
    brain.addModule(relisi);
    brain.addLogger(relisi, "getActivities");
    brain.addLogger(relisi, "getMembranePotentials");
    brain.addLogger(relisi, "getTransferFunctionParameters");
    brain.addLogger(relisi, "getLinks");

    // simulation
    brain.run();

    System.out.println(relisi.getLinks());
  }
}