package utils;

import java.util.List;
import java.util.Random;
import java.util.Vector;

/**
 * Graphs
 * 
 * @author Mathias Linkerhand
 * @version 2011-05-11
 */
public abstract class Graphs {
  public static final double[][] grosGraphAdjacencyMatrix       = new double[][] { { 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0 }, { 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0 },
      { 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0 }, { 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0 }, { 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 0 }, { 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0 },
      { 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0 }, { 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1 }, { 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1 }, { 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1 },
      { 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0 }                      };

  public static final double[][] gros2GraphAdjacencyMatrix      = new double[][] { { 0, 1, 1, 0, 0, 0, 0, 0, 1 }, { 1, 0, 1, 1, 0, 1, 1, 0, 0 },
      { 1, 1, 0, 0, 0, 1, 0, 0, 0 }, { 0, 1, 0, 0, 1, 1, 1, 1, 1 }, { 0, 0, 0, 1, 0, 1, 0, 1, 1 }, { 0, 1, 1, 1, 1, 0, 1, 1, 0 },
      { 0, 1, 0, 1, 0, 1, 0, 0, 0 }, { 0, 0, 0, 1, 1, 1, 0, 0, 0 }, { 1, 0, 0, 1, 1, 0, 0, 0, 0 } };

  public static final double[][] testGraphAdjacencyMatrix       = new double[][] {
      { 0.0, -1.0, 1.0, 1.0, -1.0, 1.0, -1.0, -1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0 },
      { -1.0, 0.0, -1.0, 1.0, 1.0, 1.0, -1.0, 1.0, -1.0, 1.0, 1.0, 1.0, 1.0, 1.0 },
      { 1.0, -1.0, 0.0, 1.0, 1.0, 1.0, -1.0, 1.0, -1.0, 1.0, -1.0, 1.0, 1.0, -1.0 },
      { 1.0, 1.0, 1.0, 0.0, 1.0, 1.0, -1.0, 1.0, -1.0, 1.0, 1.0, -1.0, 1.0, 1.0 },
      { -1.0, 1.0, 1.0, 1.0, 0.0, -1.0, 1.0, 1.0, 1.0, -1.0, -1.0, 1.0, 1.0, 1.0 },
      { 1.0, 1.0, 1.0, 1.0, -1.0, 0.0, 1.0, -1.0, -1.0, 1.0, 1.0, 1.0, -1.0, 1.0 },
      { -1.0, -1.0, -1.0, -1.0, 1.0, 1.0, 0.0, 1.0, 1.0, -1.0, 1.0, -1.0, 1.0, 1.0 },
      { -1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 1.0, -1.0, -1.0, 1.0, -1.0, -1.0 },
      { 1.0, -1.0, -1.0, -1.0, 1.0, -1.0, 1.0, 1.0, 0.0, 1.0, 1.0, 1.0, 1.0, -1.0 },
      { 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, -1.0, -1.0, 1.0, 0.0, 1.0, -1.0, 1.0, -1.0 },
      { 1.0, 1.0, -1.0, 1.0, -1.0, 1.0, 1.0, -1.0, 1.0, 1.0, 0.0, 1.0, -1.0, 1.0 },
      { 1.0, 1.0, 1.0, -1.0, 1.0, 1.0, -1.0, 1.0, 1.0, -1.0, 1.0, 0.0, -1.0, 1.0 },
      { -1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, -1.0, 1.0, 1.0, -1.0, -1.0, 0.0, 1.0 },
      { 1.0, 1.0, -1.0, 1.0, 1.0, 1.0, 1.0, -1.0, -1.0, -1.0, 1.0, 1.0, 1.0, 0.0 } };

  public static final double[][] trivialGraphAdjacencyMatrix    = new double[][] { { 0, 1, 0 }, { 1, 0, 1 }, { 0, 1, 0 } };

  public static final double[][] linkerhandGraphAdjacencyMatrix = new double[][] { { 0, 1, 1, 1, 1, 0, 0 }, { 1, 0, 1, 0, 0, 1, 1 }, { 1, 1, 0, 1, 0, 1, 0 },
      { 1, 0, 1, 0, 1, 1, 1 }, { 1, 0, 0, 1, 0, 1, 1 }, { 0, 1, 1, 1, 1, 0, 1 }, { 0, 1, 0, 1, 1, 1, 0 } };

  public static boolean isSymmetric(double[][] matrix) {
    if (matrix.length == 0) {
      return true;
    }
    if (matrix.length != matrix[0].length) {
      return false;
    }
    for (int i = 0; i < matrix.length; i++) {
      for (int j = 0; j < i; j++) {
        if (matrix[i][j] != matrix[j][i]) {
          return false;
        }
      }
    }
    return true;
  }

  public static boolean isSymmetric(List<List<Double>> matrix) {
    if (matrix.size() == 0) {
      return true;
    }
    for (int i = 0; i < matrix.size(); i++) {
      if (matrix.size() != matrix.get(i).size()) {
        return false;
      }
      for (int j = 0; j < i; j++) {
        if (!matrix.get(i).get(j).equals(matrix.get(j).get(i))) {
          return false;
        }
      }
    }
    return true;
  }

  public static double[][] createErdősRényiGraphAdjacencyMatrix(final int size, final double linkProbability, final Random random) {
    if (size < 0) {
      throw new IllegalArgumentException("Size has to be non-negative.");
    }
    if (linkProbability < 0 || linkProbability > 1) {
      throw new IllegalArgumentException("Link probability has to be ∈ [0, 1].");
    }

    // create list of all possible connections
    final int maximumLinksInUndirectedGraphOfSize = size * (size - 1) / 2;
    List<int[]> links = new Vector<int[]>(maximumLinksInUndirectedGraphOfSize);
    // create upper triangular matrix 
    for (int i = 0; i < size; i++) {
      // no self loops, so j != i
      for (int j = i + 1; j < size; j++) {
        links.add(new int[] { i, j });
      }
    }

    // remove all the links necessary randomly
    final long linksToRemove = Math.round(maximumLinksInUndirectedGraphOfSize * (1 - linkProbability));
    for (long i = 0; i < linksToRemove; i++) {
      links.remove(random.nextInt(links.size()));
    }

    // fill upper triangular matrix and symmetrize
    final double[][] adjacencyMatrix = new double[size][size];
    for (final int[] edge : links) {
      final int i = edge[0];
      final int j = edge[1];
      adjacencyMatrix[i][j] = 1.0;
      adjacencyMatrix[j][i] = 1.0; // symmetric
    }

    return adjacencyMatrix;
  }

  public static double[][] createErdosRenyiGraphAdjacencyMatrix(final int size, final double linkProbability) {
    return Graphs.createErdősRényiGraphAdjacencyMatrix(size, linkProbability, new Random());
  }

  public static double[][] getChainGraphAdjacencyMatrix(final int n) {
    if (n <= 0) {
      throw new IllegalArgumentException("n has to be positive.");
    }

    final double[][] adjacencyMatrix = new double[n][n];
    for (int i = 0; i < n; i++) {
      for (int j = 0; j < n; j++) {
        adjacencyMatrix[i][j] = Math.abs(i - j) == 1 ? 1 : 0;
      }
    }
    return adjacencyMatrix;
  }

  public static double[][] getTypicalDHAN4CliqueCircleGraphAdjacencyMatrix(final int size) {
    if (size <= 0) {
      throw new IllegalArgumentException("size has to be positive.");
    }
    if (size % 2 != 0) {
      throw new IllegalArgumentException("size has to be even.");
    }
    if(size < 12){
      throw new IllegalArgumentException("size has to be at least 12 to create only a circle without intermediate cliques due to small sizes.");
    }

    final double[][] adjacencyMatrix = new double[size][size];
    for (int i = 0; i < size; i += 2) {
      // 0 -- 1
      adjacencyMatrix[(i + 0) % size][(i + 1) % size] = 1;
      adjacencyMatrix[(i + 1) % size][(i + 0) % size] = 1;

      // 1 -- 2
      adjacencyMatrix[(i + 1) % size][(i + 2) % size] = 1;
      adjacencyMatrix[(i + 2) % size][(i + 1) % size] = 1;

      // 0 -- 2
      adjacencyMatrix[(i + 0) % size][(i + 2) % size] = 1;
      adjacencyMatrix[(i + 2) % size][(i + 0) % size] = 1;

      final int halbOpposite = (i + size / 2 + 1) % size;
      // 0 -- (1 + n/2)

      adjacencyMatrix[(i + 0) % size][halbOpposite] = 1;
      adjacencyMatrix[halbOpposite][(i + 0) % size] = 1;

      // 1 -- (1 + n/2)
      adjacencyMatrix[(i + 1) % size][halbOpposite] = 1;
      adjacencyMatrix[halbOpposite][(i + 1) % size] = 1;

      // 2 -- (1 + n/2)
      adjacencyMatrix[(i + 2) % size][halbOpposite] = 1;
      adjacencyMatrix[halbOpposite][(i + 2) % size] = 1;
    }
    return adjacencyMatrix;
  }

  public static double[][] getCircleGraphAdjacencyMatrix(final int n) {
    if (n <= 2) {
      throw new IllegalArgumentException("n has to be greater than 2.");
    }

    final double[][] adjacencyMatrix = new double[n][n];
    for (int i = 0; i < n; i++) {
      for (int j = 0; j < n; j++) {
        adjacencyMatrix[i][j] = Math.abs((i - j) % (n - 2)) == 1 ? 1 : 0;
      }
    }
    return adjacencyMatrix;
  }

  public static double[][] getTwoCliqueGraphAdjacencyMatrix(final int clique1Size, final int clique2Size, final int connections) {
    if (clique1Size < 2 || clique2Size < 2) {
      throw new IllegalArgumentException("Both cliques have to have at least to vertices.");
    }
    if (connections > clique1Size - 1 || connections > clique2Size - 1) {
      throw new IllegalArgumentException("Both cliques have to have more vertices than connections between theese cliques.");
    }
    final int size = clique1Size + clique2Size - connections;
    final double[][] twoCliqueGraphAdjacencyMatrix = new double[size][size];
    for (int i = 0; i < size; i++) {
      for (int j = 0; j < size; j++) {
        final double value;
        if (i == j) {
          // no self loops
          value = 0;
        } else if (i < clique1Size && j < clique1Size) {
          // first clique
          value = 1;
        } else if (i >= clique1Size - connections && i < clique1Size - connections + clique2Size && j >= clique1Size - connections
            && j < clique1Size - connections + clique2Size) {
          // second clique
          value = 1;
        } else {
          // no clique
          value = 0;
        }
        twoCliqueGraphAdjacencyMatrix[i][j] = value;
      }
    }

    return twoCliqueGraphAdjacencyMatrix;
  }

  public static double[][] addInhibitoryLinks(final double[][] adjacencyMatrix) {
    final double[][] result = adjacencyMatrix.clone();
    final double strength = -1.0; // negative ≙ inhibitory 
    for (int i = 0; i < result.length; i++) {
      for (int j = 0; j < result[i].length; j++) {
        if (i != j && result[i][j] == 0.0) {
          result[i][j] = strength;
        }
      }
    }
    return result;
  }
}
