package utils;

import java.io.Serializable;

/**
 * @author Mathias Linkerhand
 * @version 2010-01-25
 */
public class Interval implements Comparable<Interval>, Serializable {
  private static final long serialVersionUID                            = 1217404228388126357L;
  private double            start;
  private double            end;
  private int               comparatorMode;

  public static final int   SMALLEST_START_THEN_SMALLEST_END_IS_SMALLER = 0;
  public static final int   SMALLEST_END_THEN_SMALLEST_START_IS_SMALLER = 1;
  public static final int   SMALLEST_INTERVAL_LENGTH_IS_SMALLER         = 2;

  public Interval(final double start, final double end) {
    this(start, end, SMALLEST_START_THEN_SMALLEST_END_IS_SMALLER);
  }

  public Interval(final double start, final double end, final int comparatorMode) {
    this.start = start;
    this.end = end;
  }

  public double getEnd() {
    return this.end;
  }

  public double getStart() {
    return this.start;
  }

  @Override
  public int compareTo(final Interval interval) {
    switch (this.comparatorMode) {
      case SMALLEST_START_THEN_SMALLEST_END_IS_SMALLER:
        if (this.start != interval.start) {
          return Double.valueOf(this.start).compareTo(interval.start);
        } else {
          return Double.valueOf(this.end).compareTo(interval.end);
        }
      case SMALLEST_END_THEN_SMALLEST_START_IS_SMALLER:
        if (this.end != interval.end) {
          return Double.valueOf(this.end).compareTo(interval.end);
        } else {
          return Double.valueOf(this.start).compareTo(interval.start);
        }
      case SMALLEST_INTERVAL_LENGTH_IS_SMALLER:
        return Double.valueOf(this.end - this.start).compareTo(interval.end - interval.start);
      default:
        return 0;
    }
  }

  @Override
  public String toString() {
    return "[" + this.start + ", " + this.end + "]";
  }

  @Override
  public int hashCode() {
    final int PRIME = 31;
    int result = 1;
    long temp;
    temp = Double.doubleToLongBits(end);
    result = PRIME * result + (int) (temp ^ (temp >>> 32));
    temp = Double.doubleToLongBits(start);
    result = PRIME * result + (int) (temp ^ (temp >>> 32));
    return result;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    final Interval other = (Interval) obj;
    if (Double.doubleToLongBits(end) != Double.doubleToLongBits(other.end))
      return false;
    if (Double.doubleToLongBits(start) != Double.doubleToLongBits(other.start))
      return false;
    return true;
  }
}