package utils;

import java.io.PrintStream;
import java.util.Date;

/**
 * Evaluation of time statistics.
 * Prints progress periodically.
 * 
 * @author Mathias Linkerhand
 * @version 2010-03-09
 */
public class ProgressPrintTimer extends Timer {
  private Date        lastMessage = new Date();
  private String      name;
  private PrintStream out;
  private double      printInterval;

  public ProgressPrintTimer(final PrintStream out, final String name) {
    this.name = name;
    this.out = out;
    this.printInterval = 5 * 1000;
  }

  @Override
  public void setProgress(final double progress) {
    super.setProgress(progress);

    if (progress == 1 || new Date().getTime() - this.lastMessage.getTime() > this.printInterval) {
      this.lastMessage = new Date();
      this.out.println(this.name + ": " + ProgressPrintTimer.getProgressString(progress) + " " + ProgressPrintTimer.timeToString(this.getElapsed())
          + " elapsed, " + ProgressPrintTimer.timeToString(this.getRemaining(progress)) + " remaining");
    }
  }

  public static String getProgressString(final double progress) {
    return ProgressPrintTimer.getProgressString(progress, 40);
  }

  public static String getProgressString(final double progress, final int width) {
    String result = "[";
    for (int i = 1; i < width; i++) {
      result += i < progress * width ? "#" : "-";
    }
    result += "]";
    return result;
  }

  public static String timeToString(final long time) {
    final long days = time / (1000 * 60 * 60 * 24);
    final long hours = time / (1000 * 60 * 60) % 24;
    final long minutes = time / (1000 * 60) % 60;
    final long seconds = time / 1000 % 60;
    final long milliseconds = time % 1000;
    return (days + " d " + hours + " h " + minutes + " min " + seconds + "." + String.format("%03d", milliseconds) + " s");
  }

  public double getPrintInterval() {
    return printInterval;
  }

  public void setPrintInterval(final double printInterval) {
    this.printInterval = printInterval;
  }
}